#ifndef _LCD_H_
#define _LCD_H_

#include "types.h"
#include "mrmidi2.h"

// Text

#define TEXT_PAUSE_LEN 5

#ifndef COMPACT16x2
#ifndef NO_MENU
#define TEXT_OFF_LEN 3
#define TEXT_ON_LEN 2
#define TEXT_REC_LEN 3
#define TEXT_PLAY_LEN 4
#define TEXT_BOTH_LEN 4
#define TEXT_SONG_LEN 4
#define TEXT_DIRONCE_LEN 8
#define TEXT_SEND_LEN 4
#define TEXT_RECEIVE_LEN 4
#define TEXT_ALL_LEN 3
#if defined(RANDOMSONG) || defined(RANDOMSONG_OLD)
	#define TEXT_RANDOM_LEN 6
#endif
#if defined(RTC_NUMERIC)
	#define TEXT_CLOCK_LEN 8
	#define TEXT_DATE_LEN 8
#endif
#endif
#endif

enum display_texts_e{
	TEXT_PAUSE=0,
#ifndef COMPACT16x2
#ifndef NO_MENU
	TEXT_OFF=TEXT_PAUSE_LEN+TEXT_PAUSE,
	TEXT_ON=TEXT_OFF_LEN+TEXT_OFF,
#if defined(MIDI_MONITOR)
	TEXT_REC=TEXT_ON_LEN+TEXT_ON,
	TEXT_PLAY=TEXT_REC_LEN+TEXT_REC,
	TEXT_BOTH=TEXT_PLAY_LEN+TEXT_PLAY,
	TEXT_SONG=TEXT_BOTH_LEN+TEXT_BOTH,
#else
	TEXT_SONG=TEXT_ON_LEN+TEXT_ON,
#endif
	TEXT_DIRONCE=TEXT_SONG_LEN+TEXT_SONG,
	TEXT_ALL=TEXT_DIRONCE_LEN+TEXT_DIRONCE,
#if defined(RANDOMSONG) || defined(RANDOMSONG_OLD)
	TEXT_RANDOM=TEXT_ALL_LEN+TEXT_ALL,
    TEXT_SEND=TEXT_RANDOM_LEN+TEXT_RANDOM,
#else
    TEXT_SEND=TEXT_ALL_LEN+TEXT_ALL,
#endif
    TEXT_RECEIVE=TEXT_SEND_LEN+TEXT_SEND,
#if defined(RTC_NUMERIC)
#if defined(RANDOMSONG) || defined(RANDOMSONG_OLD)
	TEXT_CLOCK=TEXT_RANDOM_LEN+TEXT_RANDOM,
#else
	TEXT_CLOCK=TEXT_ALL_LEN+TEXT_ALL,
#endif
	TEXT_DATE=TEXT_CLOCK_LEN+TEXT_CLOCK,
#endif
#endif
#endif
	MAX_TEXT
};

enum display_strings_e{
	DISP_BLANK,
	DISP_MAIN,
	DISP_MIDICTR,
	//	DISP_BOOT,
#ifndef DIR_SUPPORT
	DISP_OK,
#endif
#ifdef THROUGH
    DISP_MIDITHROUGH,
#endif
	DISP_CARDERR,
	DISP_FILEERR,
	DISP_SELECT,
	//	DISP_PLAY,
	//	DISP_PAUSE,
#ifndef NO_RECORD
	DISP_RECRD,
	DISP_REC,
	DISP_BUFERR,
#endif
	DISP_TEMPO,
	DISP_NOFILE,
#ifndef NO_RECORD
    DISP_PREDELETE,
	DISP_DELETE,
#endif
#ifdef CALIBRATE
	DISP_CAL1,
	DISP_CAL2,
	DISP_CAL3,
	DISP_CAL4,
#endif
#ifdef SD_TIMEOUT
	DISP_SDTIMEOUT,
#endif
#ifndef NO_MENU
	DISP_MENU,
#ifdef TRANSPOSE
	DISP_MDRUM,
#endif
#ifdef LYRICS
	DISP_MLYR,
#endif
#ifdef OLD_CARDSCAN
	DISP_MSSECT,
#endif
#ifdef CHANNEL_MUTE
	DISP_MMUTE,
#endif
	DISP_MREP,
#ifdef OPTION_PAUSENEXT
	DISP_PAUSENEXT,
#endif
#ifdef BACKLIGHT
	DISP_MLIGHT,
#endif
#ifdef MIDI_MONITOR
	DISP_MIDISIG,
#endif
#ifdef MANUAL_CALIBRATE
	DISP_MMANCAL,
#endif
#ifdef CALIBRATE
	DISP_MCAL,
#endif
#ifdef USE_RTC
	DISP_SETCLOCK,
	DISP_SETDATE,
#endif
#ifdef TRACK_DELAY
	DISP_TDELAY,
#endif
#else
#ifdef CLOCK_ADJUST
	DISP_CLK,
#endif
#endif
#ifdef EXT_SPEEDSIG
    DISP_EXTCTRL,
    DISP_BPM,
    DISP_MINBPM,
    DISP_VORLAUF,
#endif
    DISP_THROUGH,
#ifdef MIDICLOCK
    DISP_CLOCK,
#endif
	MAX_DISP_ELEM
};

#define TEXTCNT	MAX_DISP_ELEM	//27

//#define COMPACT16x2

#ifdef COMPACT16x2
	#define LINESIZE	16
	#define MAX_LINES	2

	#define LINE_MAIN 0
	#define LINE_BOOT 1
	#define LINE_OK 1
	#define LINE_THROUGH 1
	#define LINE_CARDERR 1
	#define LINE_FILEERR 1
#ifdef TEMPO_IN_BPM
	#define TEMPO_X 2
	#define TEMPO_Y 1
#else
    #define TEMPO_X 2
    #define TEMPO_Y 1
#endif
	#define LINE_PLAY 0
	#define LINE_TEMPO 1
	#define PLAYFNUM_X 13
	#define PLAYFNUM_Y 0
	#define DWAHL_X 12
	#define DWAHL_Y 0
	#define STOPFNUM_X 2
	#define STOPFNUM_Y 1
	#define STOPSECT_X 12
	#define STOPSECT_Y 1
	#define LINE_PAUSE 0
	#define LINE_MAIN 0
	#define LINE_SELECT 1
	#define NAME_LINE 0
	#define FCNT_X 6
	#define FCNT_Y 1
	#define FSIZE_X 11
	#define FSIZE_Y 1
	#define LINE_NOFILE 1
	#define LINE_MENU 1
	#define LINE_OPTION 0
	#define DRUMCH_X 14
	#define LYR_X 9
	#define SSECT_X 10
	#define REP_X 13
	#define PAUSENEXT_X 14
	#define LIGHT_X 12
	#define MANCAL_X 13
    #define MANBPM_X 13
    #define MIDIMON_X 13
	#define TDELAY_X 14
	#define CLOCK_X 8
	#define DATE_X 8
	#define TRANS_X 7
	#define TRANS_Y 1
	#define MUTE_X 0
	#define MUTE_Y 1
	#define TIME_X 11
	#define TIME_Y 1
	#define LINE_RECRD 0
	#define RECFNUM_X 13
	#define RECFNUM_Y 0
    #define DIR_POS_X 13
	#define LINE_REC_BLANK 1
	#define LINE_REC 0
	#define LINE_DELETE 1
	#define LINE_BUFERR 1
    #define EXTCTRL_X 15
    #define MINBPM_X 13
    #define LEADING_X 13
	#define BOOTFNUM_X 13
	#define BOOTFNUM_Y 1
	#define BOOTSECT_X 12
	#define BOOTSECT_Y 0
    #define MIDICLK_X 11
    #define BLIP_X 15
    #define BLIP_Y 0
#else
	#define LINESIZE	24
	#define MAX_LINES	2

	#define LINE_MAIN 0
	#define LINE_BOOT 1
	#define LINE_OK 1
	#define LINE_THROUGH 1
   	#define LINE_CARDERR 1
	#define LINE_FILEERR 1
#ifdef TEMPO_IN_BPM
    #define TEMPO_X 2
    #define TEMPO_Y 1
#else
    #define TEMPO_X 6
    #define TEMPO_Y 1
#endif
	#define LINE_PLAY 0
	#define LINE_TEMPO 1
	#define PLAYFNUM_X 21
	#define PLAYFNUM_Y 0
	#define DWAHL_X 20
	#define DWAHL_Y 0
	#define STOPFNUM_X 5
	#define STOPFNUM_Y 1
	#define STOPSECT_X 14
	#define STOPSECT_Y 1
	#define LINE_PAUSE 0
	#define LINE_MAIN 0
	#define LINE_SELECT 1
	#define NAME_LINE 0
	#define FCNT_X 9
	#define FCNT_Y 1
	#define FSIZE_X 19
	#define FSIZE_Y 1
	#define LINE_NOFILE 1
	#define LINE_MENU 1
	#define LINE_OPTION 0
	#define DRUMCH_X 17
	#define LYR_X 18
	#define SSECT_X 17
	#define REP_X 11
	#define PAUSENEXT_X 15
	#define LIGHT_X 14
	#define MANCAL_X 19
    #define EXTCTRL_X 21
    #define MANBPM_X 17
    #define MINBPM_X 16
    #define LEADING_X 19
    #define MIDIMON_X 17
	#define CLOCK_X 15
	#define DATE_X 15
	#define TDELAY_X 16
	#define TRANS_X 15
	#define TRANS_Y 1
	#define MUTE_X 8
	#define MUTE_Y 0
	#define TIME_X 19
	#define TIME_Y 1
	#define LINE_RECRD 0
	#define RECFNUM_X 21
	#define RECFNUM_Y 0
    #define DIR_POS_X 19
	#define LINE_REC_BLANK 1
	#define LINE_REC 0
	#define LINE_DELETE 1
	#define LINE_BUFERR 1
	#define BOOTFNUM_X 21
	#define BOOTFNUM_Y 1
	#define BOOTSECT_X 20
	#define BOOTSECT_Y 0
    #define MIDICLK_X 11
    #define BLIP_X 18
    #define BLIP_Y 1
#endif

// Commands
#define LCD_CLEAR 0x01
#define LCD_HOME 0x02
#define LCD_MODE 0x04
#define LCD_ON 0x08
#define LCD_SHIFT 0x10
#define LCD_FUNCSET 0x20
#define LCD_CGRAM 0x40
#define LCD_DDRAM 0x80

// Functions
void lcd_data(u08up d);
void lcd_command(u08up c);
void lcd_clear(void);
void lcd_string(u08up idx, u08up row);
void lcd_print(u08up start, u08up len);
void lcd_setcur(u08up x, u08up y);
void lcd_init(void);
void lcd_nibble(u08up d);
void lcd_hex_u08(u08up d);
void lcd_hex_u16(u16up d);
//void lcd_hex_u32(u32up d);
void lcd_number(u08up d, u08up n);
void lcd_number_k(u16up d);
void lcd_number_3(u16up d);

#endif
