// Mr.Midi 2

// Hardware:
// ATmega168,88,8,328
// Uses UART for MIDI-IO
// SPI-IO plus extra Pin (CS) for SD-card
// Control buttons on PIND2-7, PINB0,1
// LCD on PORTC
// optional IR remote on PD2

// Circuit Description:
/*
Take the circuit from Mr.Midi2
available on www.mikrocontroller.net!
*/

// Software:
// MMC-Driver from alp_mp3 - Open Source Atmel AVR based MP3 Player (modified, though)
// My own MIDI-File-Parser (Format 0 only)
// My own FAT16 filesystem driver
// Compiles with WinAVR plus AVRStudio 4


/*TODO
- Record bleibt am Schluss hngen bei USE_RTC
- [opt.] Playlists als virtuelle Ordner
- Noten bleiben bei manchen Midis liegen - siehe Wo-ist-so0.mid UPDATE:defektes Midifile
- Recording mit Metronom und Taktart/Tempo
- Aufnahmepuffer mit Timestamps im Interrupt
- Play beginnt mit Pause
- Midimonitor geht nicht mehr
*/

#include "mrmidi2.h"
#include "main.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#ifndef NO_MENU
    #include <avr/eeprom.h>
#endif
#if defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__) || defined(__AVR_ATmega328__) || defined(__AVR_ATmega328P__)
    #include <avr/wdt.h>
#endif
#include "lcd.h"
#include "mmc.h"
#include "delay.h"
#include "fat16.h"
#include "types.h"

#if defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__) || defined(__AVR_ATmega328__) || defined(__AVR_ATmega328P__)
#ifdef RC5
    #include "rc5.h"
#endif

#ifdef USE_RTC
    #include "clock.h"
#endif

//ATmega168 - Umbenennungen
#define UDR UDR0
#define UCSRA UCSR0A
#define UCSRB UCSR0B
#define UCSRC UCSR0C
#define UBRRL UBRR0L
#define UBRRH UBRR0H
#define UDRE UDRE0
#define RXEN RXEN0
#define TXEN TXEN0
#define RXCIE RXCIE0
#define UDRIE UDRIE0
#define OCR2 OCR2A
#define TCCR0 TCCR0B
#define TCCR2 TCCR2B
/*#define SIG_OUTPUT_COMPARE2 SIG_OUTPUT_COMPARE2A
#define SIG_UART_RECV SIG_USART_RECV
#define SIG_UART_DATA SIG_USART_DATA
*/
#endif

#ifndef F_CPU
    #define F_CPU 8000000UL
#endif

// RTC Timer 2
#define RTC_PRESCALER           256
#if defined(RANDOMSONG) && defined(REAL_TIME_DT)
    #define RTC_RELOAD_TIMER    255
#else
    #define RTC_RELOAD_TIMER    250
#endif
#define RTC_SOFT_COUNT          125
#if F_CPU != (RTC_PRESCALER*RTC_RELOAD_TIMER*RTC_SOFT_COUNT) && !defined(REAL_TIME_DT)
    #warning RTC_NOT_ADJUSTED
#endif

#ifdef CLOCK_ADJUST
    #define CLKOUTPORT  PORTD
    #define CLKMASK     0x02 // TxD pin
#endif

#ifdef MIDI_MONITOR
    #define MAX_TICK    5
    #define MAX_DECAY   7
#endif


#if defined(CALIBRATE) || defined(MANUAL_CALIBRATE)
    #define DEFAULT_OSCCAL_POS    32
#endif


// States
enum states_e {
    STOP=       0,
    PLAY=       1,
#ifndef NO_RECORD
    REC=        2,
    ABORT_REC=  3,
#ifndef KLINGER_VERSION
    DELETE=     4,
#endif
#endif
#ifndef NO_MENU
    MENU=       5,
#endif
#ifdef TRACK_DELAY
    DELAY=      6,
#endif
    ERR=        0x80
};


#ifndef NO_MENU
// Menu
enum menu_entries_e {
#ifdef TRANSPOSE
    M_DRUMCH,
#endif
#ifdef LYRICS
    M_LYR,
#endif
#ifdef CHANNEL_MUTE
    M_MUTE,
#endif
    M_REP,
#ifdef OPTION_PAUSENEXT
	M_PAUSENEXT,
#endif
#ifdef BACKLIGHT
    M_LIGHT,
#endif
#ifdef MIDI_MONITOR
    M_MIDIMON,
#endif
#ifdef MANUAL_CALIBRATE
    M_MANCAL,
#endif
#ifdef CALIBRATE
    M_CAL,
#endif
#ifdef USE_RTC
    M_SETCLOCK,
    M_SETDATE,
#endif
#ifdef TRACK_DELAY
    M_TDELAY,
#endif
#ifdef EXT_SPEEDSIG
    M_EXTCTRL,
    M_EXTBPM,
    M_MINBPM,
    M_LEADING,
#endif
#ifdef THROUGH
    M_THROUGH,
#endif
#ifdef MIDICLOCK
    M_CLOCK,
#endif
    MAX_MENU
};
#endif

// Flags
enum flags_e {
    KEY_FLAG=    0x01,
#ifdef PLAYLIST
    PLAYLIST_FLAG=0x02,
#endif
    OCCHNG_FLAG=0x04,
#ifdef RC5
    IR_FLAG=    0x08,
    IR_TGL=        0x10,
#endif
    MMCOK_FLAG=    0x80
};

enum timeflags_e {
    TIME_DISP=    0x01,
#ifdef REAL_TIME_DT
    TIME_SEC= 0x08,
#endif
#ifdef KEY_REPEAT
    TIME_KPT=    0x02,
#endif
#ifdef MIDI_BLIP
    TIME_BLIP= 0x04,
#endif
    PAUSE_RESUME= 0x10,
#ifdef MIDICLOCK
    CLOCK_PAUSE=        0x20,
    CLOCK_RESUME=       0x40,
#endif
};

// Flags2
#define NEXT_FLAG2  0x01
#define LAST_FLAG2  0x02


// UART
#define UART_BAUD_RATE    31250
//#define UART_BAUD_RATE    19200
#define RX_BUF_SIZE        256    // lasts for at least 80ms on full MIDI data load (for *really* slow MMC write sector delays and FAT allocation searches)

// Config EEPROM
#ifndef NO_MENU
struct eedata_st { // EEPROM-Daten
#ifdef CRAPPY_COMPILER
    u08 dummy;
#endif
#ifdef TRANSPOSE
    u08 drumch;
#endif
#ifdef LYRICS
    u08 lyr;
#endif
#ifdef CHANNEL_MUTE
    u16 mute;
#endif
    u08 rep;
#ifdef BACKLIGHT
    u08 light;
#endif
#ifdef MIDI_MONITOR
    u08 midimon;
#endif
#ifdef TRACK_DELAY
    u08 tdelay;
#endif
#ifdef EXT_SPEEDSIG
    u08 extctrl;
    u08 extbpm;
    u08 minbpm;
    u08 leading;
#endif
#ifdef OPTION_PAUSENEXT
	u08 pausenext;
#endif
#ifdef THROUGH
    u08 through;
#endif
#ifdef MIDICLOCK
    u08 clock;
#endif
} ee = { 1,
#ifdef TRANSPOSE
    9,
#endif
#ifdef LYRICS
    1,
#endif
#ifdef CHANNEL_MUTE
    0,
#endif
    2,
#ifdef BACKLIGHT
    1,
#endif
#ifdef MIDI_MONITOR
    1,
#endif
#ifdef TRACK_DELAY
    0,
#endif
#ifdef EXT_SPEEDSIG
    1,
    60,
    20,
    5,
#endif
#ifdef OPTION_PAUSENEXT
    0,
#endif
#ifdef THROUGH
    0,
#endif
#ifdef MIDICLOCK
    0,  // Clock
#endif
};

// EEPROM-Belegung
enum ee_assign_et {
    EE_VERSION_POS =0,
    EE_OSCCAL_POS =    sizeof(ee)+1
};
#endif

// ROM-Konstanten
#define MTHD_LEN (4+4+4+2)
prog_uchar MTHD[MTHD_LEN]={'M','T','h','d',0,0,0,6,0,0,0,1,0,240};
prog_uchar MTRK[4]="MTrk";
#define CLOCK_START 0xfa
#define CLOCK_STOP  0xfc
#define CLOCK_TICK  0xf8
#define CLOCK_CONTINUE  0xfb

// Variables
u08 sharedmem[512];
#if !defined(NO_RECORD) || defined(BUFFERED_READ) || defined(BUFFERED_READ2) || defined(LFN_USEBUF)
    u08 rx_buf[RX_BUF_SIZE], rx_rdp=0, rx_wrp=0;    // RX/TX buffer
#endif
#if defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__) || defined(__AVR_ATmega328__) || defined(__AVR_ATmega328P__)
    #define flags GPIOR0
    #define state GPIOR1
#else
    static u08 flags=KEY_FLAG;
    static u08 state=STOP;
#endif
#ifdef NO_RESET_AFTER_PLAY
    static u08 stopbtn;
#endif
u08 dspd=0;
#if defined(KEY_REMAP)
    u08 flags2=0;
#endif
u16 tempo, new_ocr1a;    // Tempo memory
#ifdef MIDICLOCK
u16 bpmcnt, ppq24;
#endif
#ifdef TEMPO_IN_BPM
    u16 master_tempo;
#else
    u16 rtempo;
#endif
u32 trk_len;    // Track length
s32 time;
#ifdef REAL_TIME_DT
    u32 rtime;
#endif
#ifdef EXT_SPEEDSIG
    u32 new_speed=0;
    u08 recalc_speed=0, new_speed_cnt=0, toggler=0, leadcnt=0, is_pause_bpm=0;
    u16 tempo_factor=1<<10, copy_speed=0, timeout_bpm=0;
    u08 tick_pos=0, tick_cnt=0;
    #define EXTSIG_ENTPRELL 2
#endif
#ifdef BACKLIGHT
    #define LIGHT_TIME 500
    volatile u08 timerflag=0;
    volatile u16 lighttimer=0;
#endif
#ifdef RANDOMSONG
    u08 randomnumber;
#endif
#ifdef LFN_SCROLLING
    u08 lfnpos;
    enum lfn_scr_et {
        LFN_SCR_STEP = 40,
        LFN_SCR_INIT = 125,
        LFN_SCR_END = 150
    } lfnscrolltime;
#endif
#if defined(PLAYLIST) || defined(LFN_SCROLLING)
    u08 lfnlen;
#endif
#ifdef PLAYLIST
    u08 plist, plistpos;
#ifdef RANDOM_IN_PLAYLIST
    u08 plistlen;
#endif
#endif
u08 time_min, time_sec, real_time=0;
#ifndef REAL_TIME_DT
    u08 limit_count=RTC_SOFT_COUNT; // RTC
#else
    #define limit_count RTC_SOFT_COUNT
#endif
#ifndef TEMPO_IN_BPM
    s08 speed;
#endif
#ifdef TRANSPOSE
    s08 transpose;
#endif
u08 file_num=0, file_cnt=0;    // File
static volatile u08 upper_fn[8], folderlvl=0;
#ifdef MIDI_BLIP
    u08 blip;
    #define MAX_BLIP 6
    #define BLIP_ASCII_ON 255 //42
    #define BLIP_ASCII_OFF 32
#endif
#ifndef NO_MENU
    u08 menu_cnt;
#endif
#ifdef CHANNEL_MUTE
    u08 s_ch=0;
#endif
#ifdef LYRICS
    u08 gl_lyr=false;
#endif
u08 sw_version;
u08 timeflag=0; // Multi-purpose Timer 2
u08 sdtimeout;
#ifdef MIDI_MONITOR
    u08 midisig[16], notecnt[16], decaycnt;
#endif
#if defined(MIDI_MONITOR) || defined(LFN_SCROLLING)
    u08 tickcnt;
#endif
#ifdef USE_RTC
    struct time_st {
        u08 gl_sek, gl_min, gl_std, gl_tag, gl_mon, gl_jahr;
    };
    extern struct time_st gl_time;
    #define gl_sek gl_time.gl_sek
    #define gl_min gl_time.gl_min
    #define gl_std gl_time.gl_std
    #define gl_tag gl_time.gl_tag
    #define gl_mon gl_time.gl_mon
    #define gl_jahr gl_time.gl_jahr
    extern const u08 TAGE_IM_MONAT[];
    #ifdef RTC_NUMERIC
        u08 clockedit;
    #endif
#endif
#ifdef ADC_TEMPO
    u16 ad_value=1024>>1;    // ADC
#endif
#if defined(BUFFERED_READ) || defined(BUFFERED_READ2)
    #define PLAYBUF_SIZE RX_BUF_SIZE
    #define PBS_EMPTY 1
    #define PBS_FULL 2
    #define playbuf rx_buf
    #define playbuf_rdp rx_rdp
    #define playbuf_wrp rx_wrp
    u08 playbuf_state;
#endif
#ifdef TEMPO_IN_BPM
    u16 gl_timeset;
#endif
#ifdef MIDI_TX_BUF
    #define MBUF_FULL_FLAG 0x02
    #define MBUF_SIZE 16
    #define MBUF_EMPTY 1
    #define MBUF_FULL 2
    u08 midibuf[MBUF_SIZE], mbuf_wrp, mbuf_rdp, mbuf_state;
    u16 overflow=0; // DEBUG
#endif
#ifdef ACCEPT_ENCRYPTION
    #define CRYPTBYTE 0x3a
    static u08 crypt=0;
#endif

extern struct fat_filedata_st fat_filedata;
extern u32 sect;
extern u08 filemode;
#ifdef DIR_SUPPORT
    extern u16 dircl;
#endif



// Local Functions
static void hw_init(void);
static void menu_save(void);
static void calc_tempo(void);
inline static void sendbyte(u08up byte);
static u08up fetchbyte(void);
static void checkbyte(u08up b);
static u32up fetch_varlen(void);
static void key_detect(void);
static void debounce(u08up mode);
static void wait_for_deltatime(void);
static void clear_time(void);
static void start_time(void);
static inline void stop_time(void);
static void send_all_ch(u08up data);
static void send_all_off(void);
static u08up getdata(void);
static void writebyte(u08up b);
static void write_varlen(u32up value);
static void write_time(void);
static void print_time(void);
static void print_pause(void);
static void print_main(void);
static void print_menu(void);
static void print_transpose(void);
static void print_playinfo(void);
static void print_tempo(void);
static void draw_mute(void);


//---------------------------------------------------


/** Initialisiert die MCU.
    Ports, Timer, Interrupts, UART
    Highlevel Funktionen:
        LCD
        MMC/SD
        IR
    Liest EEPROM-Inhalt (oder setzt die Defaultwerte)
*/
static void hw_init(void) {
    u08up version, bl512;

    // Oscillator
#ifdef __AVR_ATmega8__
//    OSCCAL = 0xA2;
#endif
#if defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__) || defined(__AVR_ATmega328__) || defined(__AVR_ATmega328P__)
    GPIOR0 = KEY_FLAG;
#endif

    // Ports
    DDRC = 0x3F;    // LCD
    //PORTC = 0x00;

#ifdef ADC_TEMPO
    DDRB = 0xC1;    // MMC & LCD enable
    PORTB = 0xFE;
#else
    DDRB = 0xC0;    // MMC
    PORTB = 0xFF;
#endif

    DDRD = 0x02;    // Input-Port, auer TxD
    PORTD = 0xFF;    // Pull-Ups aktivieren

#ifdef ADC_TEMPO
    // ADC
    ADMUX = 0x40|5;
    ADCSRA = 0xe7;    // free running, start, prescl 128
#endif

    // LCD
    lcd_init();
    lcd_string(DISP_MAIN, LINE_MAIN);
    lcd_string(DISP_MIDICTR, LINE_BOOT);

#ifndef NO_MENU
    // Read parameters from EEPROM
    version = eeprom_read_byte(EE_VERSION_POS);
    if (version == 0xff) // blank chip
        bl512 = SW_VERSION&0x80;
    else
        bl512 = version&0x80;
    lcd_setcur(LINESIZE-1, LINE_MAIN);
    if (bl512)
        lcd_data('2');
    else
        lcd_data('1');
    if ((version&0x7f) != SW_VERSION_BASE) {    // version change
        version = SW_VERSION_BASE|bl512;
        eeprom_write_byte(EE_VERSION_POS, version);
        menu_save();
    }
    else {    // load parameters
        eeprom_read_block(&ee, (u08 *)1, sizeof(struct eedata_st));
#ifdef TRANSPOSE
        if (ee.drumch > 15)    // invalid channel
            ee.drumch = 9;
#endif
        if (ee.rep >
#if defined(RANDOMSONG) || defined(RANDOMSONG_OLD)
        4
#else
        3
#endif
        ) ee.rep = 2;
#ifdef MIDI_MONITOR
        if (ee.midimon > 3)
            ee.midimon = 1;
#endif
#ifdef TRACK_DELAY
        if (ee.tdelay > 9)
            ee.tdelay = 9;
#endif
#if defined(CALIBRATE) || defined(MANUAL_CALIBRATE)
        {
        u08up temp;
        temp = eeprom_read_byte((u08 *)EE_OSCCAL_POS);
        if (temp >= 128 && temp <= 200)
            OSCCAL = temp;
        }
#endif
    }
#endif
#ifdef BACKLIGHT
        if (ee.light) {
                DDRC |= PIN_BL;
                BL_ON;
        }
#endif
    sw_version = version;
    delay_ms(150);

#ifndef CLOCK_ADJUST
    // UART
    // Hilfsmakro zur UBRR-Berechnung ("Formel" laut Datenblatt)
    #define UART_UBRR_CALC(BAUD_,FREQ_) ((FREQ_)/((BAUD_)*16L)-1)
    UBRRH = (uint8_t)(UART_UBRR_CALC(UART_BAUD_RATE,F_CPU)>>8);
    UBRRL = (uint8_t)UART_UBRR_CALC(UART_BAUD_RATE,F_CPU);
    UCSRB = (1<<TXEN);
    // Midi device silence
//    send_all_off();
#endif

    //Initialisierung der MMC/SD-Karte
    MMC_hw_init();
#ifdef RC5
    rc5_init(RC5_ALL);
#endif
    sei();
    delay_ms(100);
    if (MMC_init()) {
#ifndef DIR_SUPPORT
        lcd_string(DISP_OK, LINE_OK);
#endif
        flags |= MMCOK_FLAG;
    }
    else {
        lcd_string(DISP_CARDERR, LINE_CARDERR);
    }

    // Timer 0: Button Debounce
    // Timer 1: Delta Time Timer
    // Timer 2: RTC
    OCR2 = RTC_RELOAD_TIMER;

    // Interrupts
#ifdef __AVR_ATmega8__
    // mega8
    TIMSK = 0x51|0x80;    // Tmr 1&2 Compare Match Int. on, Overflow Timer 0 on
#endif
    // mega168
#if defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__) || defined(__AVR_ATmega328__) || defined(__AVR_ATmega328P__)
//    TIMSK0 = 1;
//    TIMSK1 = 2;
    TIMSK2 = 2;
#endif

#ifdef THROUGH
//    UCSRB |= (1<<RXEN)|(1<<RXCIE);
    if (ee.through == 1)
        UCSRB |= (1<<RXEN)|(1<<TXEN)|(1<<RXCIE);
#endif
}


#ifndef NO_MENU
/** Save EEPROM data.
*/
static void menu_save(void) {
    eeprom_write_block(&ee, (u08 *)1, sizeof(struct eedata_st));
#if defined(CALIBRATE) || defined(MANUAL_CALIBRATE)
    eeprom_write_byte((u08 *)EE_OSCCAL_POS, OSCCAL);
#endif
}
#endif


#ifndef CLOCK_ADJUST

//----------------------------------------------------------
// UART

/** Send UART byte.
    @param Byte to send
*/
inline static void sendbyte(u08up byte) {
    loop_until_bit_is_set(UCSRA, UDRE);
    UDR = byte;
#ifdef MIDI_BLIP
    blip = MAX_BLIP;
#endif
}

#ifdef MIDI_TX_BUF
static inline void startTX(void) {
    if (!(UCSRB & (1<<UDRIE)))
        UCSRB |= (1<<UDRIE);
}

static inline void stopTX(void) {
    if (UCSRB & (1<<UDRIE))
        UCSRB &= ~(1<<UDRIE);
}

static void buf_sendbyte(u08up b)
{
    while (mbuf_state == MBUF_FULL);
    cli();
    midibuf[mbuf_wrp++] = b;
#if MBUF_SIZE != 256
    mbuf_wrp &= MBUF_SIZE-1;
#endif
    if (mbuf_wrp == mbuf_rdp)
        mbuf_state = MBUF_FULL;
    else
        mbuf_state = 0;
    sei();
}
#endif


/** Sends to all channels.
    @param Midi byte to send to all channels
*/
static void send_all_ch(u08up data) {
    u08up i;
    for (i=0; i<16; i++) {
        sendbyte(0xB0+i);
        sendbyte(data);
        sendbyte(0x00);
    }
}

#ifdef SAVE_NOTE_OFF
#if 1
volatile u16 notemem[128]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
static void send_notes_off() {
    u08 ch,note;
    for (ch=0; ch<16; ch++) {
        //sendbyte(0x80|i); // channel---
        for (note=0; note<128; note++) {
            if (notemem[note] & (u16)((u16)1 << (u16)ch)) {
				sendbyte(0x80|ch); // channel+++
                sendbyte(note);
                sendbyte(0);
            }
        }
    }
}
static inline void regnote(u08 ch, u08 note) {
    notemem[note] |= (u16)((u16)1 << (u16)ch);
}
static inline void unregnote(u08 ch, u08 note) {
    notemem[note] &= (u16)~((u16)1 << (u16)ch);
}
#else
static u16 notemem[128]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
static void send_notes_off() {}
static inline void regnote(u08 ch, u08 note) {}
static inline void unregnote(u08 ch, u08 note) {}
#endif

#endif


#ifdef PANIC_NOTEOFF
#define CHANNEL_START 1
#define CHANNEL_END 2
void send_notes_off()
{
    u08 ch,note;
    for(ch=CHANNEL_START-1; ch<CHANNEL_END; ch++)
    for (note=0; note<128; note++)
    {
        sendbyte(0x80|ch);
        sendbyte(note);
        sendbyte(0);
    }
}
#endif


/** All midi off.
    Sends midi all note off command.
*/
static void send_all_off(void) {
    send_all_ch(0x7B); // All Notes Off
#ifdef SEND_ALWAYS_CTRL_OFF
    send_all_ch(0x79); // All Controllers Off
#else
    send_all_ch(0x78); // All Sounds Off
#endif
#if defined(SAVE_NOTE_OFF) || defined(PANIC_NOTEOFF)
	send_notes_off();
#endif
}


#ifdef MIDI_MONITOR

/** Initialises all MIDI Monitor variables.
*/
static void init_midimon(void) {
    u08up i;
    for (i=0; i<16; i++) {
        midisig[i] = 0;
        notecnt[i] = 0;
    }
    tickcnt = 0;
    decaycnt = 0;
}

/** Prints the MIDI Monitor on LCD.
*/
static void print_midimon(void) {
    if (tickcnt >= MAX_TICK) {
        u08up i;
        tickcnt = 0;
        decaycnt++;
        if (decaycnt >= MAX_DECAY) {
            decaycnt = 0;
            for (i=0; i<16; i++)
                if (midisig[i])
                    midisig[i]--;
        }
        lcd_setcur(0, 1);
        for (i=0; i<16; i++)
            lcd_data(midisig[i]);
    }
}
#endif


#ifndef NO_RECORD

/** Wait for UART RX.
    @return Returns received byte
*/
static u08up getdata(void) {
    while (rx_rdp == rx_wrp && state == REC) {
        key_detect();
#ifdef MIDI_MONITOR
        if (ee.midimon&1) {
            print_midimon();
        }
#ifdef COMPACT16x2
        else
#endif
#endif
        if (timeflag&TIME_DISP)
            print_time();
    }
    u08up b = rx_buf[rx_rdp++];
#if RX_BUF_SIZE != 256
    rx_rdp &= RX_BUF_SIZE-1;
#endif
    return b;
}
#endif


//----------------------------------------------------------
// MMC IO

// Read from MMC
#ifdef BUFFERED_READ

/** Abspielpuffer fllen.
    Liest MMC-Daten und fllt damit den Puffer ganz voll.
*/
static void fill_playbuf(void) {
    if (playbuf_state == PBS_FULL)
        return;
    while ((playbuf_rdp != playbuf_wrp || playbuf_state == PBS_EMPTY)) {
        playbuf[playbuf_wrp++] = mmc_fetch_byte();
#if PLAYBUF_SIZE != 256
        playbuf_wrp &= PLAYBUF_SIZE-1;
#endif
        playbuf_state = 0;
    }
    playbuf_state = PBS_FULL;
}

/** Abspielpuffer initialisieren.
*/
static void init_playbuf(void) {
    playbuf_wrp = 0;
    playbuf_rdp = 0;
    playbuf_state = PBS_EMPTY;
}

/** Ein Byte vom Puffer holen.
    Wenn der Puffer leer ist, wird er neu gefllt.
*/
static u08up read_playbuf(void) {
    u08up temp;

    if (playbuf_state == PBS_EMPTY)
        fill_playbuf();
    temp = playbuf[playbuf_rdp++];
#if PLAYBUF_SIZE != 256
    playbuf_rdp &= PLAYBUF_SIZE-1;
#endif
    if (playbuf_rdp == playbuf_wrp)
        playbuf_state = PBS_EMPTY;
    else
        playbuf_state = 0;
    return temp;
}

/** Ein Byte von MMC lesen.
    @return Gelesenes Byte
*/
static u08up fetchbyte(void) {
    if (trk_len)
        trk_len--;
#ifdef ACCEPT_ENCRYPTION
    if (crypt) return read_playbuf()^CRYPTBYTE;
#endif
    return read_playbuf();
}

#else

/** Ein Byte von MMC lesen.
    @return Gelesenes Byte
*/
#ifdef BUFFERED_READ2
static u08up fetchbyte(void) {
    trk_len--;
    if (playbuf_rdp == 0) {
        mmc_read_block(playbuf, 256);
    }
#if PLAYBUF_SIZE == 256
#ifdef ACCEPT_ENCRYPTION
    if (crypt) return playbuf[playbuf_rdp++]^CRYPTBYTE;
#endif
    return playbuf[playbuf_rdp++];
#else
    {
        u08up temp = playbuf[playbuf_rdp++];
        playbuf_rdp &= PLAYBUF_SIZE-1;
#ifdef ACCEPT_ENCRYPTION
    if (crypt) return temp^CRYPTBYTE;
#endif
        return temp;
    }
#endif
}
#else
static u08up fetchbyte(void) {
    trk_len--;
#ifdef ACCEPT_ENCRYPTION
    if (crypt) return mmc_fetch_byte()^CRYPTBYTE;
#endif
    return mmc_fetch_byte();
}
#endif

#endif


#ifndef NO_RECORD

/** Write byte to MMC.
    @param Byte to write
*/
static void writebyte(u08up b) {
    trk_len++;
    mmc_write_byte(b);
}

/** Write Word to MMC.
    Big Endian.
    @param Word to write
*/
static void mmc_write_word(u16up b) {
    writebyte((u08up)(b>>8));
    writebyte((u08up)b);
}

/** Write DWord to MMC.
    Big Endian.
    @param DWord to write
*/
static void mmc_write_dword(u32up b) {
    writebyte((u08up)(b>>24));
    writebyte((u08up)(b>>16));
    writebyte((u08up)(b>>8));
    writebyte((u08up)b);
}

#endif

/** Read byte from MMC and verify value.
    If not identical, prints error message and stops playing.
    @param Byte to check MMC for
*/
static void checkbyte(u08up b) {
    u08up temp = fetchbyte();
    if (b != temp) {
        state = ERR+DISP_FILEERR;
    }
}


#ifdef PLAYLIST

/**    Sucht den im Puffer rx_buf genannten Song im Verzeichnis.
    Setzt file_num.
    @return false, wenn nicht gefunden.
*/
inline static u08up suchesong(void) {
    u08up i, c, e;
    for (i=0; i<file_cnt; i++) {
        fat_read_filedata(i);
        if (rx_rdp != lfnlen) // Namenlngenvergleich
            continue;
        e = 1;
        for (c=0; c<rx_rdp; c++) { // Nameninhaltsvergleich
            if (fat_filedata.name[c] != rx_buf[c+NAMESIZE]) {
                e = 0;
                break;
            }
        }
        if (!e)
            continue;
        else {
            file_num = i; // Gefunden
            return 1;
        }
    }
    return 0; // Nicht gefunden
}


/**    Liest den genannten Song aus der Datei. Die Playlist muss global bekannt sein!
    Der Name wird mit PLIST_NAME_OFFSET in rx_buf geschrieben und seine Lnge in rx_rdp.
    Es wird kein Unicode untersttzt!
    @param Songnummer in der Liste
    @return false, wenn kein weiterer Song vorhanden oder Song nicht im Verzeichnis
*/
static u08up readlistentry(u08up nr) {
    u16up pos=0;
    u08up c, e=1;
#ifdef RANDOM_IN_PLAYLIST
    u08up i;
#endif
    if (!fat_openfile(FILEMODE_READ, plist)) {
        e = 0; // Playlist kann nicht geffnet werden
    }
    else
#ifdef RANDOM_IN_PLAYLIST
        for (i=0; i<nr; i++) {
#else
        do {
#endif
            if (pos >= fat_filedata.len) {
                return 0; // Playlist durchgeschafft
            }
            do {
                c = mmc_fetch_byte();
                pos++;
            }
            while (c == 0x0d || c == 0x0a);
            rx_rdp = 0;
            do {
                rx_buf[NAMESIZE+rx_rdp] = c;
                rx_rdp++;
                if (rx_rdp == NAMESIZE)
                    break;
                c = mmc_fetch_byte();
                pos++;
            }
            while (c != 0x0d && c != 0x0a);
#ifdef RANDOM_IN_PLAYLIST
            if (i > plistlen)
                plistlen = i;
        }
#else
        }
        while (nr--);
#endif
    mmc_complete_read();
    fat_closefile();
    if (e)
        return suchesong();
    else
        return 0;
}



#endif



//----------------------------------------------------------
// MIDI format

#ifndef NO_RECORD

/** Write Midi variable quantity to SD.
    @param Binary U32 number
*/
static void write_varlen(u32up value) {
    u32up buffer = value & 0x7f;
    while (value >>= 7) {
        buffer <<= 8;
        buffer |= 0x80;
        buffer += (value & 0x7f);
    }
    while (1) {
        writebyte((u08)buffer);
        if (buffer & 0x80)
            buffer >>= 8;
        else
            break;
    }
}


/** Write delta time to SD.
    Resets delta time to 0.
*/
static void write_time(void) {
    write_varlen(time);
    time = 0;
}
#endif

/** Read Midi variable quantity from SD.
    @return Binary number
*/
static u32up fetch_varlen(void) {
    u32up v;
    u08up c;

    c = fetchbyte();
    v = c;
    if ((v & 0x80) > 0) {
        v &= 0x7F;
        do {
            c = fetchbyte();
            v = (v << 7) + (c & 0x7F);
        } while (c & 0x80);
    }
      return v;
}

#ifdef EXT_SPEEDSIG
static void handle_extsig()
{
    if (ee.extctrl == 1) {
        cli();
        tempo_factor = (u16)((((u32)copy_speed<<(u32)10) * (u32)ee.extbpm) / (u32)234375); /*wegen 4 pro messung*/
        sei();
    }
    lcd_setcur(TEMPO_X, TEMPO_Y);
    toggler = 1-toggler;
    if (toggler)
        lcd_data('*');
    else
        lcd_data(' ');
    if (leadcnt == 0) {
        if (is_pause_bpm) {// || copy_speed > (u16)((u32)ee.extbpm * 3906 / ee.minbpm)) {
            is_pause_bpm = 0;
//            if (TCCR1B&7) {
//                stop_time();    // Pause
//                send_all_off();//ch(0x78); // All Sounds Off
//                print_pause();
//            }
        }
        else if (!(TCCR1B&7)) {
            start_time();   // Start Time
            timeflag |= PAUSE_RESUME;
        }
        calc_tempo();
    }
    else
        print_tempo();
//              lcd_hex_u16(tempo_factor);
}
#endif


#ifndef NO_ATOMIC_TIME
static s32 atomic_time(void) {
    s32 t;
    cli();
    t = time;
    sei();
    return t;
}
#else
    #define atomic_time() time
#endif


#ifdef MIDICLOCK
static void check_clock() {
    static u08 tcnt=0;
    u16 lbpmcnt;
    cli();
    lbpmcnt = bpmcnt;
    sei();
    if (lbpmcnt >= ppq24) {
        cli();
        bpmcnt = 0;
        sei();
        sendbyte(CLOCK_TICK);
        tcnt++;
        if (tcnt==8) {
            lcd_setcur(MIDICLK_X, 1);
            lcd_data(' ');
        }
        if (tcnt==24) {
            tcnt=0;
            lcd_setcur(MIDICLK_X, 1);
            lcd_data('.');
        }
    }
}
#endif


/** Wait for delta time to expire.
    While waiting, check keys/IR and re-fill play buffer.
*/
inline static void wait_for_deltatime(void) {
#ifndef MIDI_TX_BUF
    u32up dt;

    dt = fetch_varlen();
    cli();
    time += dt;
    if (TCNT1 > OCR1A)
        TCNT1 = OCR1A-2;
    sei();
#endif
    while ((atomic_time() > 0) && state == PLAY) {
#ifdef MIDICLOCK
            if (ee.clock==1)
                check_clock();
            else if(ee.clock==2) {
                if (timeflag & CLOCK_PAUSE) {
                    timeflag &= ~CLOCK_PAUSE;
                    stop_time();    // Pause
                    send_all_off();
                    print_pause();
                }
                else if (timeflag & CLOCK_RESUME) {
                    timeflag &= ~CLOCK_RESUME;
                    start_time();
                    timeflag |= PAUSE_RESUME;
                }
            }
#endif
#ifdef EXT_SPEEDSIG
            if (ee.extctrl) {
                if (is_pause_bpm) {
                    if (TCCR1B&7) {
                        stop_time();    // Pause
#ifdef MIDICLOCK
                        if (ee.clock==1)
                            sendbyte(CLOCK_STOP);
#endif
                        send_all_off();//ch(0x78); // All Sounds Off
                        print_pause();
                    }
                }
                if (recalc_speed) {
                    recalc_speed = 0;
                    handle_extsig();
                }
            }
#endif
            key_detect();
            if (
#ifdef LYRICS
                !gl_lyr &&
#endif
                (atomic_time() > 0)
            ) {
#ifdef MIDI_MONITOR
                if (ee.midimon&2) {
                    print_midimon();
                }
#ifdef COMPACT16x2
                else
#endif
#endif
                if (timeflag & TIME_DISP) {
                    print_time();
                }
#ifdef MIDI_BLIP
                if (timeflag & TIME_BLIP) {
                    timeflag &= ~TIME_BLIP;
                    if (blip) {
                        if (blip == MAX_BLIP) {
                            lcd_setcur(BLIP_X, BLIP_Y);
                            lcd_data(BLIP_ASCII_ON);
                        }
                        blip--;
                        if (blip == 0) {
                            lcd_setcur(BLIP_X, BLIP_Y);
                            lcd_data(BLIP_ASCII_OFF);
                        }
                    }
                }
#endif
            }
#ifdef BUFFERED_READ
            fill_playbuf();
#endif
    }
}


/** Tempo bestimmen.
    Berechnet Zeitbasis zum Abspielen der Delta Times.
    Passt auch die Echtzeitanzeige-Geschwindigkeit an.
*/
static void calc_tempo(void) {
#ifdef ADC_TEMPO
    cli();
    new_ocr1a = tempo * ad_value / 1024 + (tempo >> 1) -1;
    flags |= OCCHNG_FLAG;
    TCNT2 = 0;    // TODO: Sicheres CTC benutzen!
    OCR2 = RTC_RELOAD_TIMER * ad_value / 1024 + RTC_RELOAD_TIMER / 2;
    sei();
    lcd_setcur(TEMPO_X, TEMPO_Y);
    if (dspd) {
        lcd_number(ad_value / 5 + 98, 1);
        OCR2 = OCR2>>1;
    }
    else {
        lcd_number(ad_value / 10 + 49, 1);
    }
    lcd_data('%');
#else
    if (dspd) {
        TCNT1 = 0;
        dspd = 0;
    }
#ifdef TEMPO_IN_BPM
    cli();
    new_ocr1a = tempo-1;
    flags |= OCCHNG_FLAG;
    sei();
    if (
#ifdef LYRICS
        gl_lyr ||
#endif
#ifdef MIDI_MONITOR
        (ee.midimon&2) ||
#endif
        0
    )
    {}
    else
        print_tempo();
#ifndef REAL_TIME_DT
    // Eigene Tempovorgabe ndert Abspielgeschwindigkeit
    if (master_tempo != tempo) {
        limit_count = RTC_SOFT_COUNT + (s08)((s32)((s32)((s32)tempo - (s32)master_tempo) * (s32)RTC_SOFT_COUNT) / (s32)master_tempo);
        if (real_time >= limit_count)
            real_time = limit_count-1;
    }
#endif
#else
    {
#ifdef EXT_SPEEDSIG
    u16 tempo_new;
    if (ee.extctrl != 1)
        tempo_factor = ((u32)1024*100)/(u32)(100+speed*5);
        //tempo_factor = 1024 - speed*20;
    tempo_new = (u16)((((u32)tempo*(u32)tempo_factor) >> (u32)10));
#endif
    cli();
#ifdef EXT_SPEEDSIG
    new_ocr1a = tempo_new - /*(tempo_new>>4)*speed -*/ 1;
#else
    new_ocr1a = tempo - (tempo>>4)*speed - 1;
#endif
    }
#ifdef REAL_TIME_DT
    rtempo = tempo;
#endif
    flags |= OCCHNG_FLAG;
    sei();
    if (
#ifdef LYRICS
        gl_lyr ||
#endif
#ifdef MIDI_MONITOR
        (ee.midimon&2) ||
#endif
        0
    )
    {}
    else
        print_tempo();
#ifndef REAL_TIME_DT
    limit_count = RTC_SOFT_COUNT - (RTC_SOFT_COUNT*speed)/(u16up)16;
    if (real_time >= limit_count)
        real_time = limit_count-1;
#endif
#endif
#endif
}


#ifdef TEMPO_IN_BPM
static u08up calc_bpm(void)
{
    return (u08up)((u32)60000000UL/(u32)((u32)tempo*gl_timeset));
}

/** Tempo in bpm ndern.
    @param Schritte
*/
static void tempo_chng(s08up step)
{
    u08up bpm = calc_bpm();
    if ((step > 0 && bpm < 250) || (step < 0 && bpm > 30)) {
        bpm += step;
        tempo = (u32)60000000UL/(u32)((u32)bpm*gl_timeset);
        calc_tempo();
    }
}
#endif

#endif // CLOCK_ADJUST

#ifdef CALIBRATE
/** Calibrate the RC internal OSC using MIDI IN data.
*/
static void calibrate_osc(void) {
    u08up old_osccal = OSCCAL;
    u08up delta, delta_min=255, best_osccal=old_osccal, bitcnt;
    u08up maxscan = 1, i, dsum;
    u16up temp;
    union {
        u08 byte[2];
        u16 word;
    } word_u;

    UCSR0B = 0;
    TCCR1B = 0;
    lcd_string(DISP_CAL1, 0);
    lcd_string(DISP_CAL2, 1);

    for (OSCCAL = 131; OSCCAL <= 181; OSCCAL++) {
        i = 0; dsum = 0;
        do {
            delay_ms(2);
            while (!(PIND & 1));
            TCNT1 = 0;
            PCIFR |= 0x04;
            PCMSK2 = 0x01;
            PCICR = 0x04;
            while (PCICR);
            word_u.word = TCNT1;
            if (word_u.byte[0] >= 0x80 && word_u.byte[1])
                bitcnt = word_u.byte[1]-1;
            else
                bitcnt = word_u.byte[1];
            temp = TCNT1/bitcnt;
            if (temp > 256)
                delta = temp-256;
            else
                delta = 256-temp;
            dsum += delta; i++;
        }
        while (i < maxscan);
        if (maxscan == 8)
            delta = (dsum+4)>>3; // Mittelwert
        if (delta < 10)
            maxscan = 8;
        else
            maxscan = 1;
        if (OSCCAL == old_osccal && delta < 6) {
            PCMSK2 = 0;
#ifdef NO_RECORD
            UCSRB = 1<<TXEN;
#else
            UCSRB = (1<<RXEN)|(1<<TXEN)|(1<<RXCIE);
#endif
            lcd_string(DISP_CAL3, 1); // "Not neccessary"
            delay_ms(1536);
            return;
        }
        else {
            lcd_setcur(20, 0);
            lcd_number((OSCCAL-131)<<1, 0);
            lcd_data('%');
        }
        if (delta < delta_min) {
            delta_min = delta;
            best_osccal = OSCCAL;
        }
    }
    OSCCAL = best_osccal; //+10?
    PCMSK2 = 0;
    UCSRB = (1<<RXEN)|(1<<TXEN)|(1<<RXCIE);
    if (delta_min > 6) {
        lcd_string(DISP_CAL4, 1); // "Manual adjust neccessary"
    }
    else {
        lcd_string(DISP_BLANK, 1);
    }
    delay_ms(1536);
}
#endif


//----------------------------------------------------------
// Buttons

/** Entprellen der Tasten.
    ATmega168 benutzt dafr den WDT.
    @param WDT-Modus [1=Timermodus, 0=WDT]
*/
static void debounce(u08up mode) {
    flags &= ~KEY_FLAG;
#if defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__) || defined(__AVR_ATmega328__) || defined(__AVR_ATmega328P__)
    cli();
    WDTCSR = 0x18;    // Setup WDT-Init
    WDTCSR = 0x40|mode;    // WDIE=1->Interrupt mode; 32ms Timeout
    wdt_reset();
    sei();
#else
    TCNT0 = 0;
    TCCR0 = 5;    // clk/1024
#endif
}


#ifdef DIREKTWAHL
/** Direktwahl des Songs per Zifferntasten.
    @param Taste
*/
u08 zif1=0xff;
u08 zif2=0xff;
inline static void direkt_wahl(u16up k) {
    if (file_cnt < 11) {
        if (k == 0)
            k = 10;
    }
    else {
        if (zif1 == 0xff) {
            zif1 = k;
            lcd_setcur(DWAHL_X, DWAHL_Y);
            lcd_data('-');
            lcd_data('>');
            lcd_data(k+'0');
            return;
        }
        else if (zif2 == 0xff) {
            zif2 = k;
            lcd_setcur(DWAHL_X+3, DWAHL_Y);
            lcd_data(k+'0');
            return;
        }
        else {
            k = k + zif1*10 + zif2*100;
        }
    }
    if (k <= file_cnt && k && k<255) {
        file_num = k-1;
        state = PLAY;
    }
    else {
        print_main();
    }
    zif1=0xff;
    zif2=0xff;
    if (state == PLAY) {
        {
            fat_read_filedata(file_num);
            print_filename(NAME_LINE);
            print_filesize();
        }
#ifdef DIR_SUPPORT
        if (fat_filedata.attr & ATTR_DIRECTORY)
            state = STOP;
        else
#endif
        {
            start_time();    // Start Time
        }
    }
}
#endif


#ifndef CLOCK_ADJUST

static u08up increase_fnum(void) {
    file_num++;
    if (file_num == file_cnt) {
        file_num = 0;
        return 1;
    }
    else
        return 0;
}

static void decrease_fnum(void) {
    if (file_num == 0)
        file_num = file_cnt;
    file_num--;
}

/** Check buttons.
    Tastenabfrage/IR-Auswertung.
*/
static void key_detect(void) {
    static u08 ok=0, wur=0;
#ifdef KEY_REPEAT
    static u08 kpt=0, krc=0;//!
#endif
    u08up lwur=0;
    u08up k = ((u08up)(~KEY_PORT)&KEY_MASK) | ((u08up)(~KEY_PORT2)&KEY_MASK2);
    if (!(flags & KEY_FLAG))
        return;
#ifndef KEY_REMAP
    if ((k&KEY_NEXT) && (k&KEY_LAST)) {
        k = RC5_CODE_MENU; lwur=1;
    }
    else if ((k&KEY_LEFT) && (k&KEY_RIGHT)) {
        k = RC5_CODE_DELETE; lwur=1;
    }
#ifdef MIDI_MONITOR
    else if ((k&KEY_LEFT) && (k&KEY_NEXT)) {
        k = RC5_CODE_MON; lwur=1;
    }
#endif
    else if ((k&KEY_STOP) && (k&KEY_NEXT)) {
        k = RC5_CODE_PLAYNEXT; lwur=1;
    }
    else if ((k&KEY_STOP) && (k&KEY_LAST)) {
        k = RC5_CODE_PLAYLAST; lwur=1;
    }
    else if (k&KEY_STOP)
        k = RC5_CODE_STOP;
#ifndef NO_RECORD
    else if (k&KEY_REC)
        k = RC5_CODE_REC;
#endif
    else if (k&KEY_LEFT)
        k = RC5_CODE_LEFT;
    else if (k&KEY_RIGHT)
        k = RC5_CODE_RIGHT;
    else if (k&KEY_NEXT)
        k = RC5_CODE_NEXT;
    else if (k&KEY_LAST)
        k = RC5_CODE_LAST;
    else if (k&KEY_PLAY)
        k = RC5_CODE_PLAY;
#else
    if ((k&KEY_NEXT) && (k&KEY_LAST)) {
        k = RC5_CODE_MENU; lwur=1; // auch RESET_TRANSPOSE
    }
    else if ((k&KEY_LEFT) && (k&KEY_RIGHT)) {
        k = RC5_CODE_DELETE; lwur=1; // auch RESET_TEMPO
    }
#ifdef LYRICS
    else if ((k&KEY_SHIFT) && (k&KEY_RIGHT)) {
        k = RC5_CODE_LYR; lwur=1;
    }
#endif
#ifdef MIDI_MONITOR
    else if ((k&KEY_SHIFT) && (k&KEY_LEFT)) {
        k = RC5_CODE_MON; lwur=1;
    }
#endif
#ifdef TRANSPOSE
    else if ((k&KEY_SHIFT) && (k&KEY_NEXT)) {
        k = RC5_CODE_TRANSPOSE_UP; lwur=1;
    }
    else if ((k&KEY_SHIFT) && (k&KEY_LAST)) {
        k = RC5_CODE_TRANSPOSE_DOWN; lwur=1;
    }
#endif
    else if (k&KEY_STOP)
        k = RC5_CODE_STOP;
#ifndef NO_RECORD
    else if (k&KEY_REC)
        k = RC5_CODE_REC;
#endif
    else if (k&KEY_LEFT)
        k = RC5_CODE_LEFT;
    else if (k&KEY_RIGHT)
        k = RC5_CODE_RIGHT;
    else if (k&KEY_NEXT)
        k = RC5_CODE_NEXT;
    else if (k&KEY_LAST)
        k = RC5_CODE_LAST;
    else if (k&KEY_PLAY)
        k = RC5_CODE_PLAY;
#endif
#ifdef ADC_TEMPO
    if (state == PLAY && (TCCR1B&7) && ad_value != ADC) {
        debounce();
        ad_value = ADC;
        calc_tempo();
    }
#endif
#ifdef RC5
    if (rc5.flip != -1) {
        k = rc5.code|0x80;
        if (k != ok || rc5.flip != ((flags>>4)&1) || (flags&IR_FLAG) == 0) {
            ok = k;
            k = 0;
        }
        flags |= IR_FLAG;
        if (rc5.flip)
            flags |= IR_TGL;
        else
            flags &= ~IR_TGL;
        rc5.flip = -1;
        debounce(3); // Lngeres Entprellen, weil >60ms Sendeabstand der FB
    }
#endif
    if (lwur) {
        wur=0;
#ifdef KEY_REPEAT
        krc = 0;
#endif
    }
#ifdef KEY_REPEAT
    if ((k != ok || (krc<2 && (kpt >= KEY_REP_TIME_INIT
#ifdef RC5
    || ((flags&IR_FLAG) && kpt >= KEY_REP_TIME_INIT_IR)
#endif
    )) || (krc>1 && (kpt >= KEY_REP_TIME
#ifdef RC5
    || ((flags&IR_FLAG) && kpt >= KEY_REP_TIME_IR)
#endif
    ))) && (!wur || (wur && !k)))//!
#else
    if (k != ok && (!wur || (wur && !k)))
#endif
    {
#ifdef KEY_REPEAT
        kpt = 0;
#endif
        if (k != ok
#ifdef RC5
        && !(flags&IR_FLAG)
#endif
        ) {
            debounce(1);
            ok = k;
        }
        if (ok) {
#ifdef BACKLIGHT
                        if (ee.light) {
                                cli();
                                lighttimer = LIGHT_TIME;
                                sei();
                                BL_ON;
                        }
#endif
#if defined(KEY_REPEAT) || defined(SD_TIMEOUT) || defined(RANDOMSONG) || defined(BACKLIGHT)
            TCCR2 = 6+8;
#endif
#if defined(KEY_REPEAT) || defined(SD_TIMEOUT)
            if (krc < 255)
                krc++;
#endif
#ifdef RC5_DISP // Debug - Anzeige RC5-Code der FB
    lcd_setcur(24,0);
    lcd_hex_u08(ok);
#endif
            k = ok & 0x7f;

#ifndef NO_MENU
            if (state == MENU) {
                if (k == RC5_CODE_LAST) { // Men blttern auf
                    menu_cnt--;
                    if (menu_cnt >= MAX_MENU)
                        menu_cnt = MAX_MENU-1;
                    print_menu();
                }
                else if (k == RC5_CODE_NEXT) { // Men blttern ab
                    menu_cnt++;
                    if (menu_cnt == MAX_MENU)
                        menu_cnt = 0;
                    print_menu();
                }
                else if (k == RC5_CODE_LEFT) { // Wert verkleinern
                    switch (menu_cnt) {
#ifdef TRANSPOSE
                        case M_DRUMCH:
                            if (ee.drumch)
                                ee.drumch--;
                            break;
#endif
#ifdef LYRICS
                        case M_LYR:
                            if (ee.lyr)
                                ee.lyr = 0;
                            break;
#endif
#ifdef CHANNEL_MUTE
                        case M_MUTE:
                            if (ee.mute & (1<<s_ch))
                                ee.mute &= ~(1<<s_ch);
                            else
                                ee.mute |= 1<<s_ch;
                            break;
#endif
                        case M_REP:
                            if (ee.rep)
                                ee.rep--;
                            break;
#ifdef BACKLIGHT
                        case M_LIGHT:
                            if (ee.light)
                                ee.light = 0;
                            break;
#endif
#ifdef MANUAL_CALIBRATE
                        case M_MANCAL:
                            if (OSCCAL > 128)
                                OSCCAL--;
                            break;
#endif
#ifdef MIDI_MONITOR
                        case M_MIDIMON:
                            if (ee.midimon)
                                ee.midimon--;
                            break;
#endif
#ifdef USE_RTC
                        case M_SETCLOCK:
                            gl_sek = 0;
                            if (gl_min && krc < FAST_SCROLL)
                                gl_min--;
                            else {
                                if (krc < FAST_SCROLL)
                                    gl_min = 59;
                                if (gl_std)
                                    gl_std--;
                                else
                                    gl_std = 23;
                            }
                            set_time();
                            break;
                        case M_SETDATE:
                            if (gl_tag > 1 && krc < FAST_SCROLL)
                                gl_tag--;
                            else {
                                if (krc < FAST_SCROLL)
                                    gl_tag = TAGE_IM_MONAT[gl_mon-1];
                                if (gl_mon > 1)
                                    gl_mon--;
                                else {
                                    gl_mon = 12;
                                    gl_jahr--;
                                }
                            }
                            set_date();
                            break;
#endif
#ifdef TRACK_DELAY
                        case M_TDELAY:
                            if (ee.tdelay)
                                ee.tdelay--;
                            break;
#endif
#ifdef EXT_SPEEDSIG
                        case M_EXTCTRL:
                            if (ee.extctrl)
                                ee.extctrl--;
                            break;
                        case M_EXTBPM:
                            if (ee.extbpm)
                                ee.extbpm--;
                            break;
                        case M_MINBPM:
                            if (ee.minbpm)
                                ee.minbpm--;
                            break;
                        case M_LEADING:
                            if (ee.leading)
                                ee.leading--;
                            break;
#endif
#ifdef OPTION_PAUSENEXT
                        case M_PAUSENEXT:
							if (ee.pausenext)
								ee.pausenext=0;
							break;
#endif
#ifdef THROUGH
                        case M_THROUGH:
                            if (ee.through){
                                ee.through=0;
                                UCSRB &= ~((1<<RXEN)|(1<<RXCIE));
                            }
                            break;
#endif
#ifdef MIDICLOCK
                        case M_CLOCK:
                            if (ee.clock)
                                ee.clock--;
                            break;
#endif
                    }
                    print_menu();
                }
                else if (k == RC5_CODE_RIGHT) { // Wert vergrern
                    switch (menu_cnt) {
#ifdef TRANSPOSE
                        case M_DRUMCH:
                            if (ee.drumch < 0x0f)
                                ee.drumch++;
                            break;
#endif
#ifdef LYRICS
                        case M_LYR:
                            if (!ee.lyr)
                                ee.lyr = 1;
                            break;
#endif
#ifdef CHANNEL_MUTE
                        case M_MUTE:
                            s_ch++;
                            s_ch &= 0x0f;
                            break;
#endif
                        case M_REP:
                            if (ee.rep <
#if defined(RANDOMSONG) || defined(RANDOMSONG_OLD)
                                4
#else
                                3
#endif
                                ) ee.rep++;
                            break;
#ifdef BACKLIGHT
                        case M_LIGHT:
                            if (!ee.light)
                                ee.light = 1;
                            break;
#endif
#ifdef CALIBRATE
                        case M_CAL:
                            calibrate_osc();
                            break;
#endif
#ifdef MANUAL_CALIBRATE
                        case M_MANCAL:
                            if (OSCCAL < 200)
                                OSCCAL++;
                            break;
#endif
#ifdef MIDI_MONITOR
                        case M_MIDIMON:
                            if (ee.midimon < 3)
                                ee.midimon++;
                            break;
#endif
#ifdef USE_RTC
                        case M_SETCLOCK:
                            gl_sek = 0;
                            if (gl_min < 60 && krc < FAST_SCROLL)
                                gl_min++;
                            else {
                                if (krc < FAST_SCROLL)
                                    gl_min = 0;
                                if (gl_std < 24)
                                    gl_std++;
                                else
                                    gl_std = 0;
                            }
                            set_time();
                            break;
                        case M_SETDATE:
                            if (gl_tag < TAGE_IM_MONAT[gl_mon-1] && krc < FAST_SCROLL)
                                gl_tag++;
                            else {
                                if (krc < FAST_SCROLL)
                                    gl_tag = 1;
                                if (gl_mon < 12)
                                    gl_mon++;
                                else {
                                    gl_mon = 1;
                                    gl_jahr++;
                                }
                            }
                            set_date();
                            break;
#endif
#ifdef TRACK_DELAY
                        case M_TDELAY:
                            if (ee.tdelay < 9)
                                ee.tdelay++;
                            break;
#endif
#ifdef EXT_SPEEDSIG
                        case M_EXTCTRL:
                            if (ee.extctrl < 2)
                                ee.extctrl++;
                            break;
                        case M_EXTBPM:
                            if (ee.extbpm < 255)
                                ee.extbpm++;
                            break;
                        case M_MINBPM:
                            if (ee.minbpm < 255)
                                ee.minbpm++;
                            break;
                        case M_LEADING:
                            if (ee.leading < 30)
                                ee.leading++;
                            break;
#endif
#ifdef OPTION_PAUSENEXT
                        case M_PAUSENEXT:
                            if (!ee.pausenext)
	                            ee.pausenext = 1;
                            break;
#endif
#ifdef THROUGH
                        case M_THROUGH:
                            if (!ee.through){
                                ee.through=1;
                                UCSRB |= (1<<RXEN)|(1<<TXEN)|(1<<RXCIE);
                            }
                            break;
#endif
#ifdef MIDICLOCK
                        case M_CLOCK:
                            if (ee.clock < 2)
                                ee.clock++;
                            break;
#endif
                    }
                    print_menu();
                }
#ifdef USE_RTC
#ifdef RTC_NUMERIC
                else if (k <= 9) { // Ziffern
                    u08up temp;
                    temp = clockedit;
                    if (menu_cnt == M_SETCLOCK) {
                        if (temp == 0) {
                            lcd_setcur(CLOCK_X+temp+1, LINE_OPTION);
                            gl_std=0; gl_min=0; gl_sek=0;
                            lcd_print(TEXT_CLOCK, TEXT_CLOCK_LEN); //lcd_data('_');lcd_data(':');lcd_data('_');lcd_data('_');lcd_data(':');lcd_data('_');lcd_data('_');
                        }
                        lcd_setcur(CLOCK_X+temp, LINE_OPTION);
                        lcd_data('0'+k);
                        if (temp == 0 || temp == 3 || temp == 6)
                            dspd = k*10;
                        else
                            dspd += k;
                        temp++;
                        if (temp == 2 || temp == 5)
                            temp++;
                        if (temp == 3)
                            gl_std = dspd;
                        else if (temp == 6)
                            gl_min = dspd;
                        else if (temp == 8) {
                            gl_sek = dspd;
                            set_time();
                        }
                        clockedit = temp;
                    }
                    else if (menu_cnt == M_SETDATE) {
                        if (temp == 0) {
                            lcd_setcur(DATE_X+temp+1, LINE_OPTION);
                            gl_tag=1; gl_mon=1; gl_jahr=7;
                            lcd_print(TEXT_DATE, TEXT_DATE_LEN); //lcd_data('_');lcd_data('.');lcd_data('_');lcd_data('_');lcd_data('.');lcd_data('_');lcd_data('_');
                        }
                        lcd_setcur(DATE_X+temp, LINE_OPTION);
                        lcd_data('0'+k);
                        if (temp == 0 || temp == 3 || temp == 6)
                            dspd = k*10;
                        else
                            dspd += k;
                        temp++;
                        if (temp == 2 || temp == 5)
                            temp++;
                        if (temp == 3)
                            gl_tag = dspd;
                        else if (temp == 6)
                            gl_mon = dspd;
                        else if (temp == 8) {
                            gl_jahr = dspd;
                            set_date();
                        }
                        clockedit = temp;
                    }
                }
#endif
#endif
                else if (k == RC5_CODE_STOP
#ifndef NO_RECORD
                    || k == RC5_CODE_REC
#endif
                    )
                { // Schlieen und speichern
                    print_main();
                    menu_save();
                }
                else if (k == RC5_CODE_MENU) { // Schlieen und verwerfen
                    print_main();
                }
            }
#endif
#ifndef KEY_REMAP
#ifndef NO_MENU
            else
#endif
            if (file_cnt
#ifndef NO_RECORD
                && state != REC
#endif
            ) {

            // Play
                if (state == PLAY) {
                    if (k == RC5_CODE_STOP) { // Stopp
                        state = STOP;
                        stopbtn = 1;
#ifdef PLAYLIST
                        if (flags & PLAYLIST_FLAG) {
                            flags &= ~PLAYLIST_FLAG;
                            file_num = plist;
                            print_main();
                        }
#endif
                    }
                    else if (k == RC5_CODE_PLAY) { // Pause
                        if (TCCR1B&7) {
                            stop_time();    // Pause
#ifdef MIDICLOCK
                            if (ee.clock==1)
                               sendbyte(CLOCK_STOP);
#endif
                            send_all_off();//ch(0x78); // All Sounds Off
                            print_pause();
                        }
                        else {
                            start_time();    // Start Time
#ifdef MIDICLOCK
                            if (ee.clock==1)
                                sendbyte(CLOCK_CONTINUE);
#endif
                            timeflag |= PAUSE_RESUME;
                        }
                    }
                    else if (k == RC5_CODE_LEFT) { // Langsamer
#ifndef TEMPO_IN_BPM
                        if (speed > -19) {
                            speed--;
                            calc_tempo();
                        }
#else
                        /* Tempo in bpm umrechnen
                         * bpm dekrementieren, falls noch mglich
                         * Neues Tempo aus bpm berechnen und anwenden
                         * */
                        tempo_chng(-1);
#endif
                    }
                    else if (k == RC5_CODE_RIGHT) { // Schneller
                        dspd = 1;
#ifndef TEMPO_IN_BPM
                        if (speed < 60) {
                            speed++;
                            calc_tempo();
                        }
#else
                        /* Tempo in bpm umrechnen
                         * bpm inkrementieren, falls noch mglich
                         * Neues Tempo aus bpm berechnen und anwenden
                         * */
                        tempo_chng(+1);
#endif
                    }
                    else if (k == RC5_CODE_DELETE) { // Tempo normal
#ifndef TEMPO_IN_BPM
                        if (speed != 0) {
                            speed = 0;
                            calc_tempo();
                        }
#else
                        if (master_tempo != tempo) {
                            tempo = master_tempo;
                            calc_tempo();
                        }
#endif
                    }
#ifdef TRANSPOSE
                    else if (k == RC5_CODE_LAST) { // Transponieren abwrts
                        if (transpose > -24) {
                            transpose--;
                            print_transpose();
                            send_all_off();
                        }
                    }
                    else if (k == RC5_CODE_NEXT) { // Transponieren aufwrts
                        if (transpose < 24) {
                            transpose++;
                            print_transpose();
                            send_all_off();
                        }
                    }
                    else if (k == RC5_CODE_MENU) { // Transpose normal
                        if (transpose != 0) {
                            transpose = 0;
                            print_transpose();
                            send_all_off();
                        }
                    }
#endif
#ifdef MIDI_MONITOR
                    else if (k == RC5_CODE_MON) {   // Monitor bei Play toggeln
                        if (ee.midimon&2) {
                            ee.midimon &= ~2;
                            print_playinfo();
                        }
                        else {
                            ee.midimon |= 2;
                            tickcnt = 0;
                            decaycnt = 0;
                            lcd_string(DISP_BLANK, LINE_TEMPO);
                        }
                    }
#endif
                }
#ifdef TRACK_DELAY
                else if (state == DELAY) {
                    if (k == RC5_CODE_PLAY) {
                        state = PLAY;
                    }
                    else if (k == RC5_CODE_STOP) {
                        print_main();
                    }
                }
#endif

            // Stop
                else if (state == STOP) {
                    if (k == RC5_CODE_PLAY
#ifdef DIR_SUPPORT
                        && !(fat_filedata.attr & ATTR_DIRECTORY)
#endif
                        )
                    { // Wiedergabe
                        state = PLAY;
                    }
#ifdef DIR_SUPPORT
                    else if (k == RC5_CODE_RIGHT && (fat_filedata.attr & ATTR_DIRECTORY)) { // Verzeichnis betreten
                        if (fat_enter_dir()) {
                            upper_fn[folderlvl&7] = file_num;
                            folderlvl++;
                            file_num = tmM_min(1, file_cnt);
                            print_main();
                        }
                    }
                    else if (k == RC5_CODE_LEFT) { // Verzeichnis verlassen
                        if (fat_leave_dir()) {
                            folderlvl--;
                            file_num = upper_fn[folderlvl&7];
                            print_main();
                        }
                    }
#endif
#ifndef NO_RECORD
                    else if (k == RC5_CODE_REC) { // Aufnahme
                        state = REC;
                    }
#endif
#ifdef DIREKTWAHL
                    else if (k <= 9) { // Ziffernwahl
                        direkt_wahl(k);
                    }
#endif
#ifndef KLINGER_VERSION
#ifndef NO_RECORD
                    else if (k == RC5_CODE_DELETE) { // Datei lschen
#ifdef DIR_SUPPORT
                        if (!(fat_filedata.attr & ATTR_DIRECTORY) && dircl == 0)
#endif
                        state = DELETE;
                    }
#endif
#endif // KLINGER_VERSION
#ifndef NO_MENU
                    else if (k == RC5_CODE_MENU) { // Men ffnen
                        menu_cnt = 0;
                        print_menu();
                        state = MENU;
                    }
#endif
#ifdef SWITCH_NAV_DIR
                    else if (k == RC5_CODE_NEXT) { // Blttern vorwrts
#else
                    else if (k == RC5_CODE_LAST) { // Blttern rckwrts
#endif
                        if (!file_num)
                            file_num = file_cnt;
                        file_num--;
                        lcd_setcur(STOPFNUM_X, STOPFNUM_Y);
                        lcd_number(file_num+1, 1);
                        {
                            fat_read_filedata(file_num);
                            print_filename(NAME_LINE);
                            print_filesize();
                        }
                    }
#ifdef SWITCH_NAV_DIR
                    else if (k == RC5_CODE_LAST) { // Blttern rckwrts
#else
                    else if (k == RC5_CODE_NEXT) { // Blttern vorwrts
#endif
                        file_num++;
                        if (file_num == file_cnt)
                            file_num = 0;
                        lcd_setcur(STOPFNUM_X, STOPFNUM_Y);
                        lcd_number(file_num+1, 1);
                        {
                            fat_read_filedata(file_num);
                            print_filename(NAME_LINE);
                            print_filesize();
                        }
                    }
                    else if (k == RC5_CODE_PLAYNEXT) { // Skip zum nchsten Song
                        file_num++;
                        if (file_num == file_cnt)
                            file_num = 0;
                        {
                            fat_read_filedata(file_num);
                            print_filename(NAME_LINE);
                            print_filesize();
                        }
                        lcd_string(DISP_BLANK, 1);
                        state = PLAY;
                    }
                    else if (k == RC5_CODE_PLAYLAST) { // Skip zum vorigen Song
                        if (file_num == 0)
                            file_num = file_cnt;
                        file_num--;
                        {
                            fat_read_filedata(file_num);
                            print_filename(NAME_LINE);
                            print_filesize();
                        }
                        lcd_string(DISP_BLANK, 1);
                        state = PLAY;
                    }
                }
            }
            else {

            // No files on card or state is REC
                if (state == STOP) {
#ifndef NO_MENU
                    if (k == RC5_CODE_MENU) { // Men ffnen
                        menu_cnt = 0;
                        print_menu();
                        state = MENU;
                    }
#endif
#ifndef NO_RECORD
#ifndef NO_MENU
                    else
#endif
                    if (k == RC5_CODE_REC && (flags & MMCOK_FLAG)) { // Aufnahme
                        state = REC;
                    }
#endif
                }

            // Rec
#ifndef NO_RECORD
                else if (state == REC) {
                    if (k == RC5_CODE_STOP) {
                        state = STOP;
                    }
                    else if (k == RC5_CODE_REC) {
                        state = ABORT_REC;
                    }
#ifdef MIDI_MONITOR
                    else if (k == RC5_CODE_MON) {   // Monitor bei Rec toggeln
                        if (ee.midimon&1) {
                            ee.midimon &= ~1;
                            lcd_string(DISP_BLANK, LINE_TEMPO);
                        }
                        else {
                            ee.midimon |= 1;
                            tickcnt = 0;
                            decaycnt = 0;
                        }
                    }
#endif
                }
#endif
            }
#else // KEY_REMAP
            else if (file_cnt
#ifndef NO_RECORD
                && state != REC
#endif
            ) {
#ifdef SWITCH_NAV_DIR
                if (k == RC5_CODE_LAST) {
#else
                if (k == RC5_CODE_LAST) { // Blttern rckwrts
#endif
                    if (state == PLAY) {
                        state = STOP;
#ifdef NO_RESET_AFTER_PLAY
                        stopbtn = 1;
#endif
                        flags2 |= LAST_FLAG2;
                    }
                    else {
                        decrease_fnum();
#ifdef TRACK_DELAY
                        if (state == DELAY) {
                            //print_main();
                            state = PLAY;
                        }
                        else
#endif
                        print_main();
                    }
                }
#ifdef SWITCH_NAV_DIR
                else if (k == RC5_CODE_NEXT) {
#else
                else if (k == RC5_CODE_NEXT) { // Blttern vorwrts
#endif
                    if (state == PLAY) {
                        state = STOP;
#ifdef NO_RESET_AFTER_PLAY
                        stopbtn = 1;
#endif
                        flags2 |= NEXT_FLAG2;
                    }
                    else {
                        increase_fnum();
#ifdef TRACK_DELAY
                        if (state == DELAY) {
                            state = PLAY;
                        }
                        else
#endif
                        print_main();
                    }
                }

            // Play
                else if (state == PLAY) {
                    if (k == RC5_CODE_STOP) { // Stopp
                        state = STOP;
#ifdef NO_RESET_AFTER_PLAY
                        stopbtn = 1;
#endif
#ifdef PLAYLIST
                        if (flags & PLAYLIST_FLAG) {
                            flags &= ~PLAYLIST_FLAG;
                            file_num = plist;
                            print_main();
                        }
#endif
                    }
                    else if (k == RC5_CODE_PLAY) { // Pause
                        if (TCCR1B&7) {
                            stop_time();    // Pause
                            send_all_off();//ch(0x78); // All Sounds Off
                            print_pause();
                        }
                        else {
                            start_time();   // Start Time
                            timeflag |= PAUSE_RESUME;
#ifdef EXT_SPEEDSIG
                            leadcnt = 0;
#endif
                        }
                    }
                    else if (k == RC5_CODE_LEFT) { // Langsamer
#ifndef TEMPO_IN_BPM
#ifdef EXT_SPEEDSIG
                        if (ee.extctrl != 1)
#endif
                        {
                            if (speed > -19) {
                                speed--;
                                calc_tempo();
                            }
                        }
#else
                        /* Tempo in bpm umrechnen
                         * bpm dekrementieren, falls noch mglich
                         * Neues Tempo aus bpm berechnen und anwenden
                         * */
                        tempo_chng(-1);
#endif
                    }
                    else if (k == RC5_CODE_RIGHT) { // Schneller
                        dspd = 1;
#ifndef TEMPO_IN_BPM
#ifdef EXT_SPEEDSIG
                        if (ee.extctrl != 1)
#endif
                        {
                            if (speed < 60) {
                                speed++;
                                calc_tempo();
                            }
                        }
#else
                        /* Tempo in bpm umrechnen
                         * bpm inkrementieren, falls noch mglich
                         * Neues Tempo aus bpm berechnen und anwenden
                         * */
                        tempo_chng(+1);
#endif
                    }
                    else if (k == RC5_CODE_DELETE) { // Tempo normal
#ifndef TEMPO_IN_BPM
                        if (speed != 0) {
                            speed = 0;
                            calc_tempo();
                        }
#else
                        if (master_tempo != tempo) {
                            tempo = master_tempo;
                            calc_tempo();
                        }
#endif
                    }
#ifdef TRANSPOSE
                    if (k == RC5_CODE_TRANSPOSE_DOWN) { // Transponieren abwrts
                        if (transpose > -24) {
                            transpose--;
                            print_transpose();
                            send_all_off();
                        }
                    }
                    else if (k == RC5_CODE_TRANSPOSE_UP) { // Transponieren aufwrts
                        if (transpose < 24) {
                            transpose++;
                            print_transpose();
                            send_all_off();
                        }
                    }
#endif
#ifdef MIDI_MONITOR
                    else if (k == RC5_CODE_MON) {   // Monitor bei Play toggeln
                        if (ee.midimon&2) {
                            ee.midimon &= ~2;
                            print_playinfo();
                        }
                        else {
                            ee.midimon |= 2;
                            tickcnt = 0;
                            decaycnt = 0;
                            lcd_string(DISP_BLANK, LINE_TEMPO);
                        }
                    }
#endif
#ifdef LYRICS
                    else if (k == RC5_CODE_LYR) {   // Lyrics bei Play toggeln
                        if (ee.lyr) { // Lyrics werden angezeigt
                            ee.lyr = 0;
                            if (gl_lyr) {
                                gl_lyr = 0;
#ifndef LFN_USEBUF
                                print_filename(NAME_LINE);
#endif
                                print_playinfo();
                            }
                        }
                        else {
                            ee.lyr = 1;
                        }
                    }
#endif
                }

#ifdef TRACK_DELAY
                else if (state == DELAY) {
                    if (k == RC5_CODE_PLAY) {
                        state = PLAY;
                    }
                    else if (k == RC5_CODE_STOP) {
                        print_main();
                    }
                }
#endif

            // Stop
                else if (state == STOP) {
                    if (k == RC5_CODE_PLAY
#ifdef DIR_SUPPORT
                        && !(fat_filedata.attr & ATTR_DIRECTORY)
#endif
                        )
                    { // Wiedergabe
                        state = PLAY;
                    }
#ifndef NO_RECORD
                    else if (k == RC5_CODE_REC) { // Aufnahme
                        state = REC;
                    }
#ifndef KLINGER_VERSION
                    else if (k == RC5_CODE_DELETE) { // Dateilschen
#ifdef DIR_SUPPORT
                        if (!(fat_filedata.attr & ATTR_DIRECTORY) && dircl == 0)
#endif
                        state = DELETE;
                    }
#endif
#endif
#ifdef DIR_SUPPORT
                    else if (k == RC5_CODE_RIGHT && (fat_filedata.attr & ATTR_DIRECTORY)) { // Verzeichnis betreten
                        //if (fat_enter_dir()) {
                            fat_enter_dir();
                            upper_fn[folderlvl] = file_num;
                            if (folderlvl<8) folderlvl+=1;
                            file_num = tmM_min(1, file_cnt);
                            print_main();
                        //}
                    }
                    else if (k == RC5_CODE_LEFT) { // Verzeichnis verlassen
                        //if (fat_leave_dir()) {
                            fat_leave_dir();
                            if(folderlvl>0) folderlvl-=1;
                            file_num = upper_fn[folderlvl];//folderlvl&7
                            print_main();
                        //}
                    }
#endif
#ifdef DIREKTWAHL
                    else if (k <= 9) { // Ziffernwahl
                        direkt_wahl(k);
                    }
#endif
#ifndef NO_MENU
                    else if (k == RC5_CODE_MENU) { // Men ffnen
                        menu_cnt = 0;
                        print_menu();
                        state = MENU;
                    }
#endif
                }
            }
            else {

            // No files on card
#ifndef NO_RECORD
                if (state == STOP)
#endif
                {
#ifndef NO_MENU
                    if (k == RC5_CODE_MENU) { // Men ffnen
                        menu_cnt = 0;
                        print_menu();
                        state = MENU;
                    }
#endif
#ifndef NO_RECORD
                    if (k == RC5_CODE_REC && (flags & MMCOK_FLAG)) { // Aufnahme
                        state = REC;
                    }
#endif
                }
#ifndef NO_RECORD
                else if (state == REC) {
                    if (k == RC5_CODE_STOP) {
                        state = STOP;
                    }
                    else if (k == RC5_CODE_REC) {
                        state = ABORT_REC;
                    }
#ifdef MIDI_MONITOR
                    else if (k == RC5_CODE_LEFT) {   // Monitor bei Rec toggeln
                        if (ee.midimon&1) {
                            ee.midimon &= ~1;
                            lcd_string(DISP_BLANK, LINE_TEMPO);
                        }
                        else {
                            ee.midimon |= 1;
                            tickcnt = 0;
                            decaycnt = 0;
                        }
                    }
#endif
                }
#endif
            }
#endif
            wur = lwur;
        }
        else {
            wur=0;
#ifdef KEY_REPEAT
            krc = 0;
#endif
        }
    }
#ifdef KEY_REPEAT
    if (ok && kpt < 255 && (timeflag&TIME_KPT))
    {
        timeflag &= ~TIME_KPT;
        kpt++;
    }
#endif
}

#else
// CLOCK_ADJUST
static void key_detect(void) {
    static u08 ok=0;
    u08up k = ((u08)(~KEY_PORT)&KEY_MASK) | ((u08)(~KEY_PORT2)&KEY_MASK2);
    if (k&KEY_LEFT)
        k = RC5_CODE_LEFT;
    else if (k&KEY_RIGHT)
        k = RC5_CODE_RIGHT;
    if (!(flags & KEY_FLAG))
        return;
#ifdef RC5
    if (rc5.flip != -1) {
        k = rc5.code|0x80;
        if (k != ok || rc5.flip != ((flags>>4)&1) || (flags&IR_FLAG) == 0) {
            ok = k;
            k = 0;
        }
        flags |= IR_FLAG;
        if (rc5.flip)
            flags |= IR_TGL;
        else
            flags &= ~IR_TGL;
        rc5.flip = -1;
        debounce(3); // Lngeres Entprellen, weil >60ms Sendeabstand der FB
    }
#endif
    if (k != ok)
    {
        if (k != ok
#ifdef RC5
        && !(flags&IR_FLAG)
#endif
        ) {
            debounce(1);
            ok = k;
        }
        if (ok) {
            k = ok & 0x7f;
            if (k == RC5_CODE_LEFT) {
                OSCCAL--;
                print_main();
            }
            else if (k == RC5_CODE_RIGHT) {
                OSCCAL++;
                print_main();
            }
        }
    }
}
#endif


/** Anzeige Hauptbildschirm.
    Zeigt Nummer, Anzahl, Dateiname und Gre an.
*/
#ifndef CLOCK_ADJUST
static void print_main(void) {
    if (file_cnt) {
#ifdef LFN_SCROLLING
        lfnpos = 0;
        lfnscrolltime = LFN_SCR_INIT;
        tickcnt = 0;
#endif
        lcd_string(DISP_SELECT, LINE_SELECT);
        lcd_setcur(STOPFNUM_X, STOPFNUM_Y);
        lcd_number(file_num+1, 1);
        lcd_setcur(FCNT_X, FCNT_Y);
        lcd_number(file_cnt, 1);
        {
            fat_read_filedata(file_num);
            print_filename(NAME_LINE);
            print_filesize();
        }
    }
    else {
        lcd_string(DISP_MAIN, LINE_MAIN);
        if (flags & MMCOK_FLAG)
            lcd_string(DISP_NOFILE, LINE_NOFILE);
        else
            lcd_string(DISP_CARDERR, LINE_CARDERR);
    }
#ifdef THROUGH
    if (ee.through==1)
        lcd_string(DISP_MIDITHROUGH, LINE_THROUGH);
#endif
    state = STOP;
}
#else
static void print_main(void) {
    lcd_string(DISP_CLK, 1);
    lcd_setcur(MANCAL_X, 1);
    lcd_number(OSCCAL, 1);
    eeprom_write_byte(EE_OSCCAL_POS, OSCCAL);
}
#endif


#ifndef NO_MENU
/** Anzeige Men.
*/
static void print_menu(void) {
    lcd_string(DISP_MENU, LINE_MENU);
    lcd_string(DISP_MENU+1+menu_cnt, LINE_OPTION);
    lcd_setcur(0, LINE_OPTION);
    if (menu_cnt+1 < 10)
        lcd_data(menu_cnt+'1');    // Nummer anzeigen
    else {
        lcd_data('1');
        lcd_data(menu_cnt-9+'0');
        lcd_data('.');
    }
#ifdef USE_RTC
#ifdef RTC_NUMERIC
    clockedit = 0;
#endif
#endif
    switch (menu_cnt) {
#ifdef TRANSPOSE
        case M_DRUMCH:
            lcd_setcur(DRUMCH_X, LINE_OPTION);
            lcd_number(ee.drumch+1, 0);
            break;
#endif
#ifdef LYRICS
        case M_LYR:
            lcd_setcur(LYR_X, LINE_OPTION);
#ifdef COMPACT16x2
            lcd_data(ee.lyr+'0');
#else
            if (!ee.lyr) {
                lcd_print(TEXT_OFF, TEXT_OFF_LEN); //lcd_data('O');lcd_data('f');lcd_data('f');
            }
            else {
                lcd_print(TEXT_ON, TEXT_ON_LEN); //lcd_data('O');lcd_data('n');
            }
#endif
            break;
#endif
#ifdef CHANNEL_MUTE
        case M_MUTE:
            draw_mute();
            break;
#endif
        case M_REP:
            lcd_setcur(REP_X, LINE_OPTION);
#ifdef COMPACT16x2
            switch (ee.rep) {
                case 0:
                    lcd_data('O');lcd_data('f');lcd_data('f'); break;
                case 1:
                    lcd_data('S');lcd_data('n');lcd_data('g'); break;
                case 2:
                    lcd_data('D');lcd_data('i');lcd_data('r'); break;
                case 3:
                    lcd_data('A');lcd_data('l');lcd_data('l'); break;
                #if defined(RANDOMSONG) || defined(RANDOMSONG_OLD)
                case 4:
                    lcd_data('R');lcd_data('n');lcd_data('d'); break;
                #endif
            }
#else
            switch (ee.rep) {
            case 0:
                lcd_print(TEXT_OFF, TEXT_OFF_LEN); break; //lcd_data('O');lcd_data('f');lcd_data('f'); break;
            case 1:
                lcd_print(TEXT_SONG, TEXT_SONG_LEN); break; //lcd_data('S');lcd_data('o');lcd_data('n');lcd_data('g'); break;
            case 2:
                lcd_print(TEXT_DIRONCE, TEXT_DIRONCE_LEN); break; //lcd_data('D');lcd_data('i');lcd_data('r');lcd_data(' ');lcd_data('o');lcd_data('n');lcd_data('c');lcd_data('e'); break;
            case 3:
                lcd_print(TEXT_ALL, TEXT_ALL_LEN); break; //lcd_data('A');lcd_data('l');lcd_data('l'); break;
#if defined(RANDOMSONG) || defined(RANDOMSONG_OLD)
            case 4:
                lcd_print(TEXT_RANDOM, TEXT_RANDOM_LEN); break; //lcd_data('R');lcd_data('a');lcd_data('n');lcd_data('d');lcd_data('o');lcd_data('m'); break;
#endif
            }
#endif
            break;
#ifdef OPTION_PAUSENEXT
        case M_PAUSENEXT:
	        lcd_setcur(PAUSENEXT_X, LINE_OPTION);
#ifdef COMPACT16x2
			lcd_data(ee.pausenext+'0');
#else
			if (ee.pausenext) lcd_print(TEXT_ON, TEXT_ON_LEN);
			else lcd_print(TEXT_OFF, TEXT_OFF_LEN);
#endif
			break;
#endif
#ifdef BACKLIGHT
        case M_LIGHT:
            lcd_setcur(LIGHT_X, LINE_OPTION);
#ifdef COMPACT16x2
            lcd_data(ee.light+'O');
#else
            if (!ee.light) {
                lcd_print(TEXT_OFF, TEXT_OFF_LEN); //lcd_data('O');lcd_data('f');lcd_data('f');
            }
            else {
                lcd_print(TEXT_ON, TEXT_ON_LEN); //lcd_data('O');lcd_data('n');
            }
#endif
            break;
#endif
#ifdef CALIBRATE
        case M_CAL:
            break;
#endif
#ifdef MANUAL_CALIBRATE
        case M_MANCAL:
            lcd_setcur(MANCAL_X, LINE_OPTION);
            lcd_number(OSCCAL, 1);
            break;
#endif
#ifdef MIDI_MONITOR
        case M_MIDIMON:
            lcd_setcur(MIDIMON_X, LINE_OPTION);
#ifdef COMPACT16x2
            lcd_data(ee.midimon+'0');
#else
            switch (ee.midimon) {
            case 0:
                lcd_print(TEXT_OFF, TEXT_OFF_LEN); break; //lcd_data('O');lcd_data('f');lcd_data('f'); break;
            case 1:
                lcd_print(TEXT_REC, TEXT_REC_LEN); break; //lcd_data('R');lcd_data('e');lcd_data('c'); break;
            case 2:
                lcd_print(TEXT_PLAY, TEXT_PLAY_LEN); break; //lcd_data('P');lcd_data('l');lcd_data('a');lcd_data('y'); break;
            case 3:
                lcd_print(TEXT_BOTH, TEXT_BOTH_LEN); break; //lcd_data('B');lcd_data('o');lcd_data('t');lcd_data('h'); break;
            }
#endif
            break;
#endif
#ifdef USE_RTC
        case M_SETCLOCK:
            read_time();
            lcd_setcur(CLOCK_X, LINE_OPTION);
            lcd_number(gl_std, 0);
            lcd_data(':');
            lcd_number(gl_min, 0);
            lcd_data(':');
            lcd_number(gl_sek, 0);
            break;
        case M_SETDATE:
            read_time();
            lcd_setcur(DATE_X, LINE_OPTION);
            lcd_number(gl_tag, 0);
            lcd_data('.');
            lcd_number(gl_mon, 0);
            lcd_data('.');
            lcd_number(gl_jahr, 0);
            break;
#endif
#ifdef TRACK_DELAY
        case M_TDELAY:
            lcd_setcur(TDELAY_X, LINE_OPTION);
            lcd_data(ee.tdelay+'0');
            break;
#endif
#ifdef EXT_SPEEDSIG
        case M_EXTCTRL:
            lcd_setcur(EXTCTRL_X, LINE_OPTION);
#ifdef COMPACT16x2
            lcd_data(ee.extctrl+'0');
#else
            if (ee.extctrl == 1) {
                lcd_data(' ');
                lcd_print(TEXT_ON, TEXT_ON_LEN);
            }
            else
            if (ee.extctrl == 2) {
                lcd_data(' ');
                lcd_print(TEXT_PAUSE, TEXT_PAUSE_LEN);
            }
            else
                lcd_print(TEXT_OFF, TEXT_OFF_LEN);
#endif
            break;
        case M_EXTBPM:
            lcd_setcur(MANBPM_X, LINE_OPTION);
            lcd_number(ee.extbpm, 1);
            break;
        case M_MINBPM:
            lcd_setcur(MINBPM_X, LINE_OPTION);
            lcd_number(ee.minbpm, 1);
            break;
        case M_LEADING:
            lcd_setcur(LEADING_X, LINE_OPTION);
            lcd_number(ee.leading, 0);
            break;
#endif
#ifdef THROUGH
        case M_THROUGH:
            lcd_setcur(LIGHT_X, LINE_OPTION);
#ifdef COMPACT16x2
            lcd_data((ee.through==1) ? '1' : 'O');
#else
            if (ee.through==1) {
                lcd_print(TEXT_ON, TEXT_ON_LEN); //lcd_data('O');lcd_data('n');
            }
            else {
                lcd_print(TEXT_OFF, TEXT_OFF_LEN); //lcd_data('O');lcd_data('f');lcd_data('f');
            }
#endif
            break;
#endif
#ifdef MIDICLOCK
        case M_CLOCK:
            lcd_setcur(CLOCK_X, LINE_OPTION);
#ifdef COMPACT16x2
            lcd_data(ee.clock+'O');
#else
            if (ee.clock==1)
                lcd_print(TEXT_SEND, TEXT_SEND_LEN);
            else if (ee.clock==2)
                lcd_print(TEXT_RECEIVE, TEXT_RECEIVE_LEN);
            else
                lcd_print(TEXT_OFF, TEXT_OFF_LEN);
#endif
            break;
#endif
        default:
            break;
    }
}
#endif


#ifdef TRANSPOSE
/** Anzeige Transpose.
*/
static void print_transpose(void) {
    if (
#ifdef LYRICS
        gl_lyr ||
#endif
#ifdef MIDI_MONITOR
        (ee.midimon&2) ||
#endif
        0
    )
        return;
    lcd_setcur(TRANS_X, TRANS_Y);
    if (transpose < 0) {
        lcd_data('-');
        lcd_number(-transpose, 0);
    }
    else {
        lcd_data('+');
        lcd_number(transpose, 0);
    }
}
#endif


#ifdef CHANNEL_MUTE
/** Anzeige Muteleiste.
*/
static void draw_mute(void) {
    u08up i;
    lcd_setcur(MUTE_X, MUTE_Y);
    for (i=0; i<16; i++) {
        if (ee.mute & (1<<i)) {
            if (i == s_ch)
                lcd_data('X');
            else
                lcd_data('x');
        }
        else {
            if (i == s_ch)
                lcd_data('O');
            else
                lcd_data('o');
        }
    }
}
#endif


/** Special print function
    @param row
    @param col
    @param pointer to string (0-terminated)
*/
/*
static void print(u08up y, u08up x, char *s) {
    lcd_setcur(x, y);
    while (*s)
        lcd_data(*s++);
}
*/


//----------------------------------------------------------
// Time

#ifndef CLOCK_ADJUST

/** Zeit am Display anzeigen.
*/
static void print_time(void) {
    timeflag &= ~TIME_DISP;
    lcd_setcur(TIME_X, TIME_Y);
    lcd_number(time_min, 0);
    lcd_data(':');
    lcd_number(time_sec, 0);
}

/** Pause anstatt Zeit anzeigen.
*/
static void print_pause(void) {
    lcd_setcur(TIME_X, TIME_Y);
    lcd_print(TEXT_PAUSE, TEXT_PAUSE_LEN); //lcd_data('P'); lcd_data('A'); lcd_data('U'); lcd_data('S'); lcd_data('E');
}

/** Geschwindigkeit am Display anzeigen.
*/
static void print_tempo(void)
{
#ifndef EXT_SPEEDSIG
    lcd_setcur(TEMPO_X, TEMPO_Y);
#endif
#ifdef TEMPO_IN_BPM
    // Tempo aus "tempo" in bpm berechnen und anzeigen
    // tempo: us pro bpm
    u08up bpm = calc_bpm();
    lcd_number(bpm, 1);
#else
#ifdef EXT_SPEEDSIG
    lcd_setcur(TEMPO_X+1, TEMPO_Y);
    //lcd_number_k(tempo_factor);
    lcd_number_3(((u32)1024*100)/(u32)tempo_factor);
    lcd_data('%');
//    lcd_data('0'+tick_pos);
//    lcd_hex_u16(OCR1A);
#else
    if (speed < 0) {
        lcd_data('-');
        lcd_nibble(-speed);
    }
    else {
        lcd_data('+');
        lcd_nibble(speed);
    }
#endif
#endif
}

/** Transpose und Geschwindigkeit am Display anzeigen.
*/
static void print_playinfo(void)
{
    lcd_string(DISP_TEMPO, LINE_TEMPO);
#ifdef TRANSPOSE
    print_transpose();
#endif
    print_tempo();
}


/** Zeit zurcksetzen.
*/
static void clear_time(void) {
#ifndef REAL_TIME_DT
    TCCR2 = 0;
#endif
    time_sec = 0;
    time_min = 0;
    real_time = 0;
#ifndef REAL_TIME_DT
    limit_count = RTC_SOFT_COUNT;
#else
    rtime = 0;
#endif
}


/** Zeitanzeige starten.
*/
static void start_time(void) {
    print_time();
    TCNT1 = 0;
    TIFR1 = 2;
    TIMSK1 = 2;
    TCCR1B = 8+2;
#ifndef REAL_TIME_DT
    TCNT2 = 0;
    TCCR2 = 6+8;
#endif
}


/** Zeitanzeige stoppen.
*/
static inline void stop_time(void) {
    TCCR1B = 0;
    TIMSK1 = 0;
#ifndef REAL_TIME_DT
    TCCR2 = 0;
#endif
}

#endif


//----------------------------------------------------------
// Main
#ifdef RANDOMSONG

/** Zufallslied berechnen.
    Dabei wird das genannte Lied vermieden.
    @param Vermeiden
    @param Maximale Nummer
    @return Zufallszahl
*/
static u08up random_song(u08up avoid, u08up top) {
    u08up temp;
    if (top < 2)
        return 0;
    do {
        temp = TCNT2 ^ randomnumber;
        while (temp >= top)
            temp -= top;
    }
    while (temp == avoid
#ifdef DIR_SUPPORT
        || (dircl && temp == 0)
#endif
    );
    return temp;
}
#endif

/** Hauptprogramm.
*/
//void main(void) __attribute__ ((naked)); <- Achtung buggy
void main(void) {
union {
    u08 byte[4];
    u16 word[2];
    u32 udword;
} temp;
    u08up ev, len=0, adp=0, trp=0, ch=0, i, pausenext=0;
    u08up ade=0;
#ifdef CHANNEL_MUTE
    u08up ch_nact=0;
#endif
#ifdef LYRICS
    u08up lyrx, lyry;
#endif
#ifndef TEMPO_IN_BPM
    u16up gl_timeset;
#endif

#ifdef LFN_USEBUF
    fat_filedata.name = &rx_buf[0];
#endif
    hw_init();

    if (flags & MMCOK_FLAG) {
        if (!fat_init()) {
            lcd_string(DISP_CARDERR, LINE_CARDERR);
            while (1);
        }
        else {
            fat_count_files();
        }
        file_num = 0;
        print_main();
#ifdef CLOCK_ADJUST
        TCNT1 = 0;
        TCCR1B = 8+3;
        OCR1A = 62500-1; //0.5 second toggling interval
#endif
    }
#ifdef BACKLIGHT
    BL_OFF;
#endif

    while (1) {
#ifndef NO_RECORD
        if (state == REC) {
                // Rekorder-Task
                if (!fat_openfile(FILEMODE_WRITE, file_cnt)) {
                    print_main();
                    continue;
                }
                mmc_write_start(cluster2sector(fat_filedata.startcl)+1);
                OCR1A = 500000UL/240-1;    // Fixes Tempo festlegen
                trk_len = 490;
                time = 0;
                rx_wrp = 0;    // Puffer leeren
                rx_rdp = 0;
#ifdef MIDI_MONITOR
                init_midimon();
#endif

                lcd_string(DISP_RECRD, LINE_RECRD);    // Bildschirm anzeigen
                lcd_setcur(RECFNUM_X, RECFNUM_Y);
                lcd_number(file_cnt+1, 1);
                lcd_string(DISP_BLANK, LINE_REC_BLANK);

                clear_time();
#ifdef REAL_TIME_DT
                rtempo = 500000UL/240;
#endif
                UCSRB |= (1<<RXEN)|(1<<RXCIE);

//#define BACKUP_REC
#ifndef BACKUP_REC
                while (state == REC) {
                    ev = getdata();    // Auf Midi-Event warten
                    if (state == STOP || state == ABORT_REC || state >= ERR)
                        break;
                    write_time();
                    if (!TCCR1B) {
                        start_time();//TCCR1B = 8+2;    // Start Time
                        lcd_string(DISP_REC, LINE_REC);
                        lcd_setcur(RECFNUM_X, RECFNUM_Y);
                        lcd_number(file_cnt+1, 1);
                    }
                    writebyte(ev);

                    if (ev == 0xf0) {    // SysEx
                        do {
                            i = getdata();
                            writebyte(i);
                        } while (i != 0xf7);
                    }
                    else if (ev < 0x80) {    // Running status
#ifdef MIDI_MONITOR
                        if ((ade&0xf0) == 0x80) { // NoteOff
                            writebyte(getdata());
                            if (notecnt[ch])
                                notecnt[ch]--;
                            if (!notecnt[ch])
                                midisig[ch] = 1;
                        }
                        else if ((ade&0xf0) == 0x90) { // NoteOn
                            i = getdata();
                            writebyte(i);
                            i = (i>>4)+1;
                            if (i > 7) i = 7;
                            midisig[ch] = i;
                            notecnt[ch]++;
                        }
                        else
#endif
                        {
                            len = adp-1;
                            while (len) {
                                writebyte(getdata());
                                len--;
                            }
                        }
                    }
                    else {
                        // MidiEvent
                        switch (ev) {
                        case 0xf2:
                            len = 2;
                            break;
                        case 0xf3:
                            len = 1;
                            break;
                        default:
                            ch = ev & 0x0f;
                            switch (ev & 0xf0) {
                            case 0xf0:
                                len = 0;
                                break;
                            case 0xc0:    // ProgramChange
                            case 0xd0:    // ChannelAftertouch
                                len = 1;
                                break;
                            case 0x80:    // NoteOff
#ifdef MIDI_MONITOR
                                writebyte(getdata());
                                writebyte(getdata());
                                if (notecnt[ch])
                                    notecnt[ch]--;
                                if (!notecnt[ch])
                                    midisig[ch] = 1;
                                len = 0;
                                break;
#endif
                            case 0x90:    // NoteOn
#ifdef MIDI_MONITOR
                                writebyte(getdata());
                                i = getdata();
                                writebyte(i);
                                i = (i>>4)+1;
                                if (i > 7) i = 7;
                                midisig[ch] = i;
                                notecnt[ch]++;
                                len = 0;
                                break;
#endif
                            case 0xa0:    // PolyphonicAftertouch
                            case 0xb0:    // ControlModeChange
                            case 0xe0:    // PitchWheel
                                len = 2;
                                break;
                            default:
                                len = 0;
                            }
                        }
                        adp = len;
#ifdef MIDI_MONITOR
                        ade = ev;
#endif
                        while (len) {
                            writebyte(getdata());
                            len--;
                        }
                    }
                }

#else //Backup

                while (state == REC) {
                    ev = getdata();    // Auf Midi-Event warten
                    if (state == STOP || state == ABORT_REC || state >= ERR)
                        break;
                    write_time();
                    if (!TCCR1B) {
                        start_time();//TCCR1B = 8+2;    // Start Time
                        lcd_string(DISP_REC, LINE_REC);
                        lcd_setcur(RECFNUM_X, RECFNUM_Y);
                        lcd_number(file_cnt+1, 1);
                    }
                    writebyte(ev);

                    if (ev == 0xf0) {    // SysEx
                        do {
                            ev = getdata();
                            writebyte(ev);
                        } while (ev != 0xf7);
                    }
                    else if (ev < 0x80) {    // Running status
                        len = adp-1;
                        while (len > 0) {
                            writebyte(getdata());
                            len--;
                        }
                    }
                    else {
                        // MidiEvent
                        switch (ev) {
                        case 0xf2:
                            len = 2;
                            break;
                        case 0xf3:
                            len = 1;
                            break;
                        default:
                            switch (ev & 0xf0) {
                            case 0xf0:
                                len = 0;
                                break;
                            case 0xc0:    // ProgramChange
                            case 0xd0:    // ChannelAftertouch
                                len = 1;
                                break;
                            case 0x80:    // NoteOff
                            case 0x90:    // NoteOn
                            case 0xa0:    // PolyphonicAftertouch
                            case 0xb0:    // ControlModeChange
                            case 0xe0:    // PitchWheel
                                len = 2;
                                break;
                            default:
                                len = 0;
                            }
                        }
                        adp = len;
                        while (len > 0) {
                            writebyte(getdata());
                            len--;
                        }
                    }
                }
#endif
#ifdef THROUGH
//                UCSRB &= ~(1<<RXCIE);
#else
                UCSRB &= ~((1<<RXEN)|(1<<RXCIE));
#endif
                if (!TCCR1B || state == ABORT_REC || state >= ERR) {
                    stop_time();
                    mmc_complete_write();
                    filemode = FILEMODE_CLOSED; // Datei abmurksen!
                }
                else {
                    stop_time();
                    mmc_write_dword(0x00ff2f00UL);
                    gl_timeset = sect + 1;    // Nchster freier Sektor
                    mmc_complete_write();

                    mmc_write_start(cluster2sector(fat_filedata.startcl));
                    for (i=0; i<MTHD_LEN; i++)
                        mmc_write_byte((u08)pgm_read_byte( &(MTHD[i]) ));
                    for (i=0; i<4; i++)
                        mmc_write_byte((u08)pgm_read_byte( &(MTRK[i]) ));
                    mmc_write_dword(trk_len);
                    mmc_write_word(255);
                    mmc_write_byte(2);
                    write_varlen(485);
                    mmc_complete_write();

                    {
                        fat_filedata.len = trk_len + 20;
                        fat_closefile();
                    }
                    file_num = file_cnt;
                    file_cnt++;
                }

                print_main();
        }
        else
#endif

#ifndef CLOCK_ADJUST
        if (state == PLAY) {
                //u08up oade;
                // Wiedergabe starten
#ifndef THROUGH
                UCSRB &= ~((1<<RXEN)|(1<<RXCIE));
#else
                if (ee.through==1) {
                    print_main();
                    continue;
                }
                else
                    UCSRB &= ~((1<<RXEN)|(1<<RXCIE));
#endif
#ifdef ACCEPT_ENCRYPTION
                crypt = 0;
#endif
#ifdef PLAYLIST
                if (fat_filedata.name[lfnlen-3] == 'P'
                    && fat_filedata.name[lfnlen-2] == 'L'
                    && fat_filedata.name[lfnlen-1] == 'S') {
                    flags |= PLAYLIST_FLAG;
                    plistpos = 0;
#ifdef RANDOM_IN_PLAYLIST
                    plistlen = 0;
                    readlistentry(255);
#endif
                    plist = file_num;
                    if (!readlistentry(0)) { // Abort
                        print_main();
                        continue;
                    }
                }
#endif
#ifdef LYRICS
                gl_lyr = false;
                lyrx = 0; lyry = 0;
#endif
#ifdef LFN_SCROLLING
                lfnpos = 0;
#endif
#ifdef USE_ALLCTRLOFF
                send_all_ch(0x79); // All Controllers Off
#endif
                clear_time();
#ifdef TRANSPOSE
                transpose = 0;
#endif
#ifdef BACKLIGHT
                if (ee.light) {
                                        cli();
                    lighttimer=LIGHT_TIME;
                                        sei();
                    BL_ON;
                }
#endif
#ifdef MIDI_MONITOR
                if (ee.midimon&2) // Anzeige nur wenn keine Midi-Pegelanzeige
                    lcd_string(DISP_BLANK, LINE_TEMPO);
                else
#endif
                    print_playinfo();

                {
                    fat_openfile(FILEMODE_READ, file_num);
                }
#if defined(LFN_SCROLLING) || defined(PLAYLIST) || defined(TRACK_DELAY)
                print_filename(NAME_LINE);
#endif

#ifdef BUFFERED_READ
                init_playbuf();
#endif
#ifdef BUFFERED_READ2
                playbuf_rdp = 0;
#endif
#ifdef MIDI_TX_BUF
                mbuf_wrp=0; mbuf_rdp=0; mbuf_state=MBUF_EMPTY;
#endif
                ev = 1;
                for (i=0; i<4; i++)
                {
                    u08 hb = fetchbyte(), mh = (u08)pgm_read_byte( &(MTHD[i]) );
#ifdef ACCEPT_ENCRYPTION// Verschlsselte Dateien erkennen
                    if (hb != mh && (hb^CRYPTBYTE) != mh)
                        ev = 0;
                    if (ev == 1 && (hb^CRYPTBYTE) == mh)
                        ev = 2;
#else
                    if (hb != mh)
                        ev = 0;
#endif
                }
#ifdef ACCEPT_ENCRYPTION// Verschlsselte Dateien erkennen
                if (ev==2) crypt = 1;
#endif

                if (!ev) {
                    mmc_complete_read();
                    fat_closefile();
                    state = ERR+DISP_FILEERR;
                    continue; // Play abbrechen
                }

                for (i=0; i<6; i++)
                    fetchbyte();
                checkbyte(0);
                checkbyte(1);
                temp.byte[1] = fetchbyte();    // Tempo lesen
                temp.byte[0] = fetchbyte();
                gl_timeset = temp.word[0];

                TCNT1 = 0;
                // Tempo berechnen
                if (gl_timeset < 0x8000) {   // Delta-Ticks
                    tempo = 500000UL/gl_timeset;
#ifdef MIDICLOCK
                    ppq24 = gl_timeset/24;
#endif
                }
                else {               // SMPTE-Timeformat
                    tempo = 1000000UL/(-(s08)(gl_timeset>>8))/gl_timeset;
#ifdef MIDICLOCK
                    ppq24 = 500000UL/tempo/24;
#endif
                }
#ifdef TEMPO_IN_BPM
                master_tempo = tempo;
#endif
                for (i=0; i<4; i++)
                    fetchbyte(); // MTrk

                temp.byte[3] = fetchbyte();    // Lnge lesen
                temp.byte[2] = fetchbyte();
                temp.byte[1] = fetchbyte();
                temp.byte[0] = fetchbyte();
                trk_len = temp.udword;
#ifndef TEMPO_IN_BPM
                speed = 0;
#endif
#ifdef MIDI_MONITOR
                init_midimon();
#endif

#ifdef EXT_SPEEDSIG
                tempo_factor = 1<<10;
                copy_speed = 0;
#endif
                calc_tempo();
                time = 0;
                //oade = 0;
#ifdef NO_RESET_AFTER_PLAY
                stopbtn = 0;
#endif
                if (pausenext){
                    pausenext=0;
                    print_pause();
                    stop_time();    // Pause
                    while(1){
                        key_detect();
                        if (state == STOP || (TCCR1B&7))
                            break;
                    }
                    timeflag |= PAUSE_RESUME;
                }
                
                start_time();    // Start Time
#ifdef EXT_SPEEDSIG
                if (ee.extctrl) {
                    timeout_bpm = 1832/ee.minbpm;
                    TCCR0A = 0;
                    TCCR0B = 0;
                    TIMSK0 = 1;
                    /* INT0 on falling edge */
                    /* clear pending INT0 */
                    /* enable INT0 interrupt */
                    EICRA = 0x02;
                    EIFR = 0x01;
                    EIMSK |= 1;
                    leadcnt = ee.leading;
                    if (leadcnt) {
                        stop_time();    // Pause
                        print_pause();
                        while (leadcnt) {
                            key_detect();
                            if (state == STOP)
                                break;
                            if (recalc_speed) {
                                recalc_speed = 0;
                                handle_extsig();
                                leadcnt--;
                                lcd_setcur(LINESIZE-1,1);
                                lcd_data('0'+leadcnt);
                            }
                        }
                        start_time();   // Start Time
                        timeflag |= PAUSE_RESUME;
                    }
                }
#endif
#ifdef MIDICLOCK
                if (ee.clock==1) {
                    sendbyte(CLOCK_START);
                    bpmcnt = 0;
                }
#endif

                while (state == PLAY) {
#ifdef MIDI_TX_BUF
                    u32 dt;
#endif
                    if (trk_len == 0) {    // Tracklen wird heruntergezhlt. Bei 0 = Ende erreicht
                        state = STOP;
                        break;
                    }
#ifdef MIDI_TX_BUF
                    #define sendbyte(a) buf_sendbyte(a)
                    dt = fetch_varlen(); // wie wait_for_deltatime() vorderer Block!
                    cli();
                    time += dt;
                    if (TCNT1 > OCR1A)
                        TCNT1 = OCR1A-2;
                    sei();
                    if (state == STOP || state >= ERR)
                        break;
#else
                    wait_for_deltatime();
                    if (state == STOP || state >= ERR)
                        break;
                    if (TCCR1B == 0)//(flags & PAUSE_FLAG)
                    {
                        continue;
                    }
#endif
#ifdef MIDICLOCK
                    if (ee.clock==1)
                        check_clock();
#endif
                    ev = fetchbyte();
                    if (ev == 0xFF) {    // META-EVENT
                        timeflag &= ~PAUSE_RESUME;
                        ev = fetchbyte();
                        switch (ev) {
                            case 0x2f:    // End Of Track
                                checkbyte(0);
                                trk_len = 0;
                                break;
                            case 0x51:    // Tempo Change
                                checkbyte(3);
                                temp.byte[3] = 0;
                                temp.byte[2] = fetchbyte();
                                temp.byte[1] = fetchbyte();
                                temp.byte[0] = fetchbyte();
                                TCNT1 = 0;
                                tempo = temp.udword/gl_timeset;
#ifdef TEMPO_IN_BPM
                                master_tempo = tempo;
#endif
                                calc_tempo();
                                break;
#ifdef LYRICS
                            //case 0x01:    // Text
                            case 0x05:    // Lyrics
                                if (ee.lyr) {
                                    len = 0;
                                    gl_lyr = true;
                                    temp.word[0] = fetch_varlen();
                                    lcd_setcur(lyrx, lyry);
                                    while (temp.word[0]) {
                                        temp.word[0]--;
                                        ev = fetchbyte();
                                        if (ev >= ' ' && ev <= '~') {
                                            lcd_data(ev);
                                            lyrx++;
                                            if (lyrx == LINESIZE) {
                                                lyrx = 0;
                                                lyry++;
                                                if (lyry == MAX_LINES)
                                                    lyry = 0;
                                                lcd_string(DISP_BLANK, lyry);
                                                lcd_setcur(0, lyry);
                                            }
                                        }
                                    }
                                    break;
                                }
                                // Break fehlt hier absichtlich!
#endif
                            default:
                                temp.word[0] = fetch_varlen();
                                while (temp.word[0]) {
                                    temp.word[0]--;
                                    fetchbyte();
                                }
                        }
                    }
                    else if (ev == 0xf0) {    // SYSEX
                        sendbyte(0xf0);
                        timeflag &= ~PAUSE_RESUME;
                        temp.word[0] = fetch_varlen();
                        while (temp.word[0]) {
                            temp.word[0]--;
                            ev = fetchbyte();
                            sendbyte(ev);
                        }
                        //oade = 0;
                    }
                    else if (ev >= 0x80) {    // MIDI-EVENT
                        trp = false;
                        timeflag &= ~PAUSE_RESUME;
#ifdef CHANNEL_MUTE
                        ch_nact = 1;
#endif
                        switch (ev) {
                            case 0xf2:
                                len = 2;
                                break;
                            case 0xf3:
                                len = 1;
                                break;
                            default:
                                ch = ev & 0x0f;
#ifdef CHANNEL_MUTE
                                ch_nact = 0;
#endif
                                switch (ev & 0xf0) {
                                    case 0xf0:
                                        len = 0;
                                        break;
                                    case 0xc0:    // ProgramChange
                                    case 0xd0:    // ChannelAftertouch
                                        len = 1;
                                        break;
                                    case 0x80:    // NoteOff
#ifdef MIDI_MONITOR
                                        if (notecnt[ch])
                                            notecnt[ch]--;
                                        if (!notecnt[ch])
                                            midisig[ch] = 1;
#endif
                                    case 0x90:    // NoteOn
                                    case 0xa0:    // PolyphonicAftertouch
                                        trp = true;
                                    case 0xb0:    // ControlModeChange
                                    case 0xe0:    // PitchWheel
                                        len = 2;
                                        break;
                                    default:
                                        state = ERR+DISP_FILEERR;
                                }
                        }
                        adp = len;    // Lnge merken (fr Running Status)
#ifdef CHANNEL_MUTE
                        if (ch_nact || !(ee.mute & ((u16up)1<<ch)))
#endif
                        {
//X2                            if (oade != ev) { // Eigener Running Status spart Bandbreite
                                sendbyte(ev);
//X2                                oade = ev;
//X2                            }
                            if (trp) {
                                u08 note;
				note = fetchbyte();
#ifdef TRANSPOSE
                                if (ch != ee.drumch)
                                    note += transpose;
#endif
                                sendbyte(note);

                                i = fetchbyte();
                                sendbyte(i);
#ifdef SAVE_NOTE_OFF
                                if ((ev&0xf0) == 0x90) { // NoteOn
                                    if (i>0) regnote(ch, note);
                                    else unregnote(ch, note);
                                }
                                else if ((ev&0xf0) == 0x80) { // NoteOff
                                    unregnote(ch, note);
                                }
#endif
#ifdef MIDI_MONITOR
                                if ((ev&0xf0) == 0x90) { // NoteOn
                                    i = (i>>4)+1;
                                    if (i > 7) i = 7;
                                    midisig[ch] = i;
                                    notecnt[ch]++;
                                }
#endif
                            }
                            else {
#ifdef MIDI_MONITOR
                                if ((ev&0xf0) == 0xB0) { // ControlMode AllNotesOff/AllSoundsOff auf midisig mappen
                                    i = fetchbyte();
                                    sendbyte(i);
                                    if (i == 0x78 || i == 0x7B) {
                                        midisig[ch] = 1;
                                        notecnt[ch] = 0;
                                    }
                                    sendbyte(fetchbyte());
                                }
                                else
#endif
                                while (len) {
                                    sendbyte(fetchbyte());    // Bytes durchschleifen
                                    len--;
                                }
                            }
                        }
#ifdef CHANNEL_MUTE
                        else
                        {
                            while (len) {
                                fetchbyte();    // Bytes ignorieren
                                len--;
                            }
                        }
#endif
                        ade = ev;
                    }
                    else {            // Running Status
                        if (timeflag & PAUSE_RESUME) {
                            timeflag &= ~PAUSE_RESUME;
                            sendbyte(ade);
                        }
#ifdef CHANNEL_MUTE
                        if (ch_nact || !(ee.mute & ((u16up)1<<ch)))
#endif
                        {
#ifdef TRANSPOSE
                            if (trp && ch != ee.drumch)
                                sendbyte(ev+transpose);
                            else
#endif
                                sendbyte(ev);
#ifdef MIDI_MONITOR
                            if ((ade&0xf0) == 0x90) { // NoteOn
                                i = fetchbyte();
                                sendbyte(i);
                                i = (i>>4)+1;
                                if (i > 7) i = 7;
                                midisig[ch] = i;
                                notecnt[ch]++;
                            }
                            else if ((ade&0xf0) == 0x80) { // NoteOff
                                i = fetchbyte();
                                sendbyte(i);
                                if (notecnt[ch])
                                    notecnt[ch]--;
                                if (!notecnt[ch])
                                    midisig[ch] = 1;
                            }
                            else if ((ade&0xf0) == 0xB0) {
                                i = fetchbyte();
                                sendbyte(i);
                                if (ev == 0x78 || ev == 0x7B) {
                                    midisig[ch] = 1;
                                    notecnt[ch] = 0;
                                }
                            }
                            else
#endif
                            {
                                len = adp - 1;
                                while (len) {
                                    sendbyte(fetchbyte());
                                    len--;
                                }
                            }
                        }
#ifdef CHANNEL_MUTE
                        else {
                            len = adp - 1;
                            while (len) {
                                fetchbyte();    // Bytes ignorieren
                                len--;
                            }
                        }
#endif
                    }
#ifdef MIDI_TX_BUF
                    wait_for_deltatime();
                    #undef sendbyte
#endif
                }
#ifdef EXT_SPEEDSIG
                TCCR0B = 0;
                EIMSK = 0;
#endif
                stop_time();
#ifdef MIDICLOCK
                if (ee.clock==1)
                    sendbyte(CLOCK_STOP);
#endif
#ifdef MIDI_TX_BUF
                while (UCSRB & (1<<UDRIE)); // complete sending buffer
#endif
//#ifdef THROUGH
//                UCSRB |= (1<<RXEN); //|(1<<RXCIE);
//#endif
                    fat_closefile();
#ifndef NO_RESET_AFTER_PLAY
                send_all_off();
#else
                if (stopbtn) {
                    send_all_off();
                    stopbtn = 0;
                }
#endif
#if defined(NO_MENU)
                if (trk_len == 0) {
                    if (file_num < file_cnt-1) {
                        file_num++;
                        state = PLAY;
                    }
                    else {
                        file_num = 0;
                        state = PLAY;
                    }
                }
#else
                i = ee.rep;
#ifdef OPTION_PAUSENEXT
				if (ee.pausenext)
                    pausenext = 1;
#endif
                ev = 0;
#if defined(RANDOMSONG) || defined(DIR_SUPPORT)
                adp = file_num;
#endif
                // repeat modes:
                // 0=single song
                // 1=repeat single song
                // 2=all songs
                // 3=repeat all songs
                // 4=random song
                if (trk_len == 0) { // Song hat zu Ende gespielt
                    if (i == 0) {}
                    else if (i == 1)
                        ev = 1;
                    else
                        ev = 4;
                }
#ifdef KEY_REMAP
                else if (flags2 & NEXT_FLAG2) { // Next/Last wurde gedrckt
                    ev = 2;
                }
                else if (flags2 & LAST_FLAG2) { // Next/Last wurde gedrckt
                    ev = 3;
                }
#endif
                if (ev) { // Weiterspielen erwnscht
#ifdef DIR_SUPPORT
                    do
#endif
                    {
#ifdef RANDOMSONG
                        if (i == 4) {
#ifdef PLAYLIST
#ifdef RANDOM_IN_PLAYLIST
                            if (flags & PLAYLIST_FLAG) {
                                adp = plistpos;
                                do
                                    plistpos = random_song(adp, plistlen);
                                while (!readlistentry(plistpos));
                            }
                            else
#endif
#endif
                            file_num = random_song(adp, file_cnt);
                        }
                        else
#endif
#ifdef KEY_REMAP
                        if (ev == 3) {
#ifdef PLAYLIST
                            if (flags & PLAYLIST_FLAG) {
                                plistpos--;
                                if (plistpos == 255 || !readlistentry(plistpos)) {
                                    flags &= ~PLAYLIST_FLAG;
                                    file_num = plist;
                                    print_main();
                                    break;
                                }
                            }
                            else
#endif
                            decrease_fnum();
                        }
                        else
#endif
                        if (i == 2 || i == 3
#ifdef KEY_REMAP
                            || ev == 2
#endif
                        ) {
#ifdef PLAYLIST
                            if (flags & PLAYLIST_FLAG) {
                                if (!readlistentry(++plistpos)) {
                                    flags &= ~PLAYLIST_FLAG;
                                    file_num = plist;
                                    print_main();
                                    break;
                                }
                            }
                            else
#endif
                            if (increase_fnum()) { // Wrap-Around tritt auf
                                if (i == 2
#ifdef KEY_REMAP
                                    && ev != 2
#endif
                                ) {
                                    decrease_fnum();
                                    print_main();
                                    break;
                                }
                            }
                        }
                        print_main();
                        state = PLAY;
                    }
#ifdef DIR_SUPPORT
                    while (ev != 1 && (fat_filedata.attr & ATTR_DIRECTORY || file_num == adp));
//                    if (file_num == adp) // Suche nach nchstem Titel war erfolglos
//                        state = STOP;
#endif
                }
                else { // Abspielende
#ifdef PLAYLIST
                    if (flags & PLAYLIST_FLAG) {
                        flags &= ~PLAYLIST_FLAG;
                        file_num = plist;
                    }
#endif
                    print_main();
                }
#ifdef TRACK_DELAY
                if (ee.tdelay && state == PLAY
#ifdef KEY_REMAP
                    && !(flags2&3)
#endif
                ) {
                    state = DELAY;
                    clear_time();
#ifndef REAL_TIME_DT
                    limit_count = RTC_SOFT_COUNT;
#else
                    time_sec = ee.tdelay;
#endif
                    TCCR2 = 8+6;
                    lcd_string(DISP_BLANK, 1);
                    print_time();
                }
#endif
#ifdef KEY_REMAP
                flags2 &= ~0x03;
#endif
#endif
        }
#endif
#ifndef KLINGER_VERSION
#ifndef NO_RECORD
        else if (state == DELETE) {
            u08up keys = ((u08up)(~KEY_PORT)&KEY_MASK) | ((u08up)(~KEY_PORT2)&KEY_MASK2);
            lcd_string(DISP_PREDELETE, LINE_DELETE);
            if (keys & KEY_PLAY)
            {
                lcd_string(DISP_DELETE, LINE_DELETE);    // Bildschirm anzeigen
                file_cnt--;
                fat_delete_file();
                if (file_num == file_cnt)
                    file_num--;
                delay_ms(1024);
                print_main();
            }
            if (keys & KEY_STOP)
            {
                print_main();
            }                
        }
#endif
#endif // KLINGER_VERSION
#ifdef TRACK_DELAY
        else if (state == DELAY) {
            key_detect();
#ifndef REAL_TIME_DT
            if (timeflag & TIME_DISP) {
                timeflag &= ~TIME_DISP;
                print_time();
            }
            if (time_sec >= ee.tdelay) {
                state = PLAY;
            }
#else
            if (timeflag & TIME_SEC) {
                timeflag &= ~TIME_SEC;
                print_time();
            }
            if (time_sec == 0) {
                state = PLAY;
            }
#endif
        }
#endif
        else if (state == STOP) {
            pausenext=0;
//#ifdef THROUGH
//            if (ee.through)// && (UCSRB & (1<<RXCIE))
//                UCSRB |= 1<<RXCIE;
//            if (ee.through && (UCSRA & 0x80)) {
//                volatile u08 temp;
//                temp = UDR;
//                UDR = temp;
//            }
//#endif
            key_detect();
#ifdef LFN_SCROLLING_NEW
#ifdef DIR_SUPPORT
#ifndef COMPACT16x2
            i = (fat_filedata.attr & ATTR_DIRECTORY)?5:0;
#else
            i = (fat_filedata.attr & ATTR_DIRECTORY)?3:0;
#endif
#endif
            if (file_cnt && lfnlen > LINESIZE
#ifdef DIR_SUPPORT
                - i
#endif
            ) { // Voraussetzung zum Scrollen
                if (tickcnt >= lfnscrolltime
#ifdef DIREKTWAHL
                    && zif1 == 0xff
#endif
                ) {
                    lfnscrolltime = LFN_SCR_STEP;
                    tickcnt = 0;
                    lfnpos++;
                    if (lfnpos >= lfnlen)
                        lfnpos = 0;
                    print_filename(NAME_LINE);
                }
            }
#elif defined(LFN_SCROLLING)
#ifdef DIR_SUPPORT
#ifndef COMPACT16x2
            i = (fat_filedata.attr & ATTR_DIRECTORY)?5:0;
#else
            i = (fat_filedata.attr & ATTR_DIRECTORY)?3:0;
#endif
#endif
            if (file_cnt && lfnlen > LINESIZE
#ifdef DIR_SUPPORT
                - i
#endif
            ) { // Voraussetzung zum Scrollen
                if (tickcnt >= lfnscrolltime
#ifdef DIREKTWAHL
                    && zif1 == 0xff
#endif
                ) {
                    tickcnt = 0;
                    if (lfnscrolltime == LFN_SCR_INIT || lfnscrolltime == LFN_SCR_STEP) {
                        lfnpos++;
                        print_filename(NAME_LINE);
                        if (lfnpos >= lfnlen - LINESIZE
#ifdef DIR_SUPPORT
                            + i
#endif
                        )
                            lfnscrolltime = LFN_SCR_END;
                        else
                            lfnscrolltime = LFN_SCR_STEP;
                    }
                    else {
                        lfnpos = 0;
                        print_filename(NAME_LINE);
                        lfnscrolltime = LFN_SCR_INIT;
                    }
                }
            }
#endif
        }
#ifndef NO_MENU
        else if (state == MENU) {
            key_detect();
        }
#endif
        else {
            stop_time();
            lcd_string(state-ERR, 1);
            delay_ms(1024);
            print_main();
        }
    }
}


//----------------------------------------------------------
// IRQ

/** Timer for delta times.
*/
ISR(TIMER1_COMPA_vect) {
#ifdef CLOCK_ADJUST
    CLKOUTPORT ^= CLKMASK;
#else
#ifdef MIDICLOCK
    if (ee.clock == 2)
        return; // kein timer bei clock receive
    bpmcnt++;
#endif
    if (state == REC)
        time++;
    else
        time--;
#ifdef MIDI_TX_BUF
    if (time <= 0 && mbuf_state != MBUF_EMPTY)
        startTX();
#endif
    if (flags & OCCHNG_FLAG) {
        flags &= ~OCCHNG_FLAG;
        OCR1A = new_ocr1a;
    }
#ifdef REAL_TIME_DT
#ifdef TEMPO_IN_BPM
    rtime += master_tempo;
#else
    rtime += rtempo;
#endif
    if (rtime >= 1000000UL) {
        rtime -= 1000000UL;
        time_sec++;
        if (time_sec == 60) {
            time_sec = 0;
            time_min++;
        }
        timeflag |= TIME_DISP;
    }
#endif
#endif
}


#ifndef CLOCK_ADJUST
/** Timer for real time clock.
*/
ISR(TIMER2_COMPA_vect) {
#if defined(MIDI_MONITOR) || defined(LFN_SCROLLING)
    tickcnt++;
#endif
#ifdef RANDOMSONG
    randomnumber++;
#endif
    if (sdtimeout) {
        sdtimeout--;
    }
#ifdef KEY_REPEAT
    timeflag |= TIME_KPT;
#endif
#ifdef MIDI_BLIP
    timeflag |= TIME_BLIP;
#endif
#ifdef BACKLIGHT
        if (ee.light) {
                if (lighttimer == 0) {
                        BL_OFF;
                }
                else {
                        BL_ON;
                        lighttimer--;
                }
        }
        else
                BL_OFF;
#endif
#ifndef REAL_TIME_DT
    real_time++;
    if (real_time == limit_count) {
        real_time = 0;
        time_sec++;
        if (time_sec == 60) {
            time_sec = 0;
            time_min++;
        }
        timeflag |= TIME_DISP;
    }
#else
#ifdef TRACK_DELAY
    if (state == DELAY) {
        real_time++;
        if (real_time == limit_count) {
            real_time = 0;
            if (time_sec)
                time_sec--;
            timeflag |= TIME_SEC;
        }
    }
#endif
#endif
}
#endif


/** Timer for debouncing buttons.
*/
#if defined(__AVR_ATmega168__) || defined(__AVR_ATmega88__) || defined(__AVR_ATmega328__) || defined(__AVR_ATmega328P__)
ISR(WDT_vect) {
    wdt_disable();
    flags |= KEY_FLAG;
#ifdef RC5
    if (flags & IR_FLAG && rc5.flip == -1)
        flags &= ~IR_FLAG;
#endif
}
#else
ISR(TIMER0_OVF_vect) {
    TCCR0 = 0;
    flags |= KEY_FLAG;
}
#endif



#ifndef NO_RECORD
/** UART RX.
*/
#ifdef THROUGH
ISR(USART_RX_vect) {
    u08 tmp = UDR;
    if (ee.through==1) {
        if (UCSRA & 0x20)
            UDR = tmp;
    }
    else {
#ifdef MIDICLOCK
        if (ee.clock == 2) {
            if (state == REC)
                return;
            else if (tmp == CLOCK_TICK) {
                time -= ppq24;
#ifdef REAL_TIME_DT
#ifdef TEMPO_IN_BPM
                rtime += master_tempo*ppq24;
#else
                rtime += rtempo*ppq24;
#endif
                if (rtime >= 1000000UL) {
                    rtime -= 1000000UL;
                    time_sec++;
                    if (time_sec == 60) {
                        time_sec = 0;
                        time_min++;
                    }
                    timeflag |= TIME_DISP;
                }
#endif
            }
            else if (tmp == CLOCK_START || tmp == CLOCK_CONTINUE)
            {
                timeflag |= CLOCK_RESUME;
            }
            else if (tmp == CLOCK_STOP)
            {
                timeflag |= CLOCK_PAUSE;
            }
        }
#endif
        if (/*state != REC ||*/ tmp == 0xf8 || tmp == 0xfe)    // Midi Clock und Synch ausblenden
            return;
        rx_buf[rx_wrp++] = tmp;
#if RX_BUF_SIZE != 256
        rx_wrp &= RX_BUF_SIZE-1;
#endif
        if (rx_wrp == rx_rdp) {    // Buffer overflow!
            state = ERR+DISP_BUFERR;
        }
    }
}
#else

ISR(USART_RX_vect) {
    u08up tmp = UDR;
    if (/*state != REC ||*/ tmp == 0xf8 || tmp == 0xfe)    // Midi Clock und Synch ausblenden
        return;
    rx_buf[rx_wrp++] = tmp;
#if RX_BUF_SIZE != 256
    rx_wrp &= RX_BUF_SIZE-1;
#endif
    if (rx_wrp == rx_rdp) {    // Buffer overflow!
        state = ERR+DISP_BUFERR;
    }
}
#endif

#endif


#ifdef CALIBRATE
/** Pinnderung fr Kalibrierung.
*/
ISR(PCINT2_vect) {
    if (!(PIND & 1)) {
        NOP;
        TCCR1B = 1;
    }
    else {
        TCCR1B = 0;
        PCICR &= ~0x04;
    }
}
#endif


#ifdef MIDI_TX_BUF
ISR(USART_UDRE_vect) {
    if (mbuf_state != MBUF_EMPTY) {
        UDR = midibuf[mbuf_rdp++];
#if MBUF_SIZE != 256
        mbuf_rdp &= MBUF_SIZE-1;
#endif
        if (mbuf_rdp == mbuf_wrp)
            mbuf_state = MBUF_EMPTY;
        else
            mbuf_state = 0;
#ifdef MIDI_BLIP
        blip = MAX_BLIP;
#endif
    }
    else
        stopTX();
}
#endif


#ifdef EXT_SPEEDSIG
u16 mytick = 0, ticks[4];
ISR(TIMER0_OVF_vect)
{
    if (mytick < 65535) {
        mytick++;
        if (mytick >= timeout_bpm)
            if (!is_pause_bpm)
                is_pause_bpm = 1;
    }
    else
        TCCR0B = 0; // Timer stop
}

ISR(INT0_vect)
{
    if (TCCR0B == 0){
        TCCR0B = (1 << CS02) | (1 << CS00);
        mytick=0;
        TCNT0=0;
        tick_cnt=0;
        tick_pos=0;
    }
    else if (mytick >= EXTSIG_ENTPRELL) {
        TCCR0B = 0; // Timer stop
        ticks[tick_pos] = (mytick<<7)|(TCNT0>>1);
        if (tick_cnt < 4) {
            tick_cnt++;
            copy_speed = ticks[tick_pos];
        }
        else {
            copy_speed = (ticks[0]+ticks[1]+ticks[2]+ticks[3]) >> 2;
        }
        tick_pos++;
        tick_pos &= 3;
        TCNT0 = 0;
        mytick = 0;
        TCCR0B = (1 << CS02) | (1 << CS00);
        recalc_speed = 1;
    }
}
#endif

